"""
Created on 24 Jan 2022

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)
"""

import optparse

from scs_analysis import version


# --------------------------------------------------------------------------------------------------------------------

class CmdCognitoDevices(object):
    """unix command line handler"""

    def __init__(self):
        """
        Constructor
        """
        self.__parser = optparse.OptionParser(usage="%prog  [-c CREDENTIALS] "
                                                    "{ -F [{ -t TAG | -n INVOICE }] [-m] "
                                                    "| -U TAG INVOICE "
                                                    "| -D TAG } "
                                                    "[-i INDENT] [-v]",
                                              version=version())

        # identity...
        self.__parser.add_option("--credentials", "-c", type="string", action="store", dest="credentials_name",
                                 help="the stored credentials to be presented")

        # operations...
        self.__parser.add_option("--Find", "-F", action="store_true", dest="find", default=False,
                                 help="list the devices visible to me")

        self.__parser.add_option("--Update", "-U", type="string", action="store", nargs=2, dest="update",
                                 help="update the device")

        self.__parser.add_option("--Delete", "-D", type="string", action="store", dest="delete",
                                 help="delete the device (superuser only)")

        # filters...
        self.__parser.add_option("--tag", "-t", type="string", action="store", dest="tag",
                                 help="filter by device tag")

        self.__parser.add_option("--invoice", "-n", type="string", action="store", dest="invoice_number",
                                 help="filter by invoice")

        # output...
        self.__parser.add_option("--memberships", "-m", action="store_true", dest="memberships", default=False,
                                 help="show device's organisation memberships")

        self.__parser.add_option("--indent", "-i", type="int", action="store", dest="indent",
                                 help="pretty-print the output with INDENT")

        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    def is_valid(self):
        count = 0

        if self.find:
            count += 1

        if self.update:
            count += 1

        if self.delete is not None:
            count += 1

        if count != 1:
            return False

        if self.memberships and not self.find:
            return False

        if self.tag and self.invoice_number:
            return False

        if self.__args:
            return False

        return True


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def credentials_name(self):
        return self.__opts.credentials_name


    @property
    def find(self):
        return self.__opts.find


    @property
    def update(self):
        return bool(self.__opts.update)


    @property
    def update_tag(self):
        return self.__opts.update[0] if self.update else None


    @property
    def update_invoice(self):
        return self.__opts.update[1] if self.update else None


    @property
    def delete(self):
        return self.__opts.delete


    @property
    def tag(self):
        return self.__opts.tag


    @property
    def invoice_number(self):
        return self.__opts.invoice_number


    @property
    def memberships(self):
        return self.__opts.memberships


    @property
    def indent(self):
        return self.__opts.indent


    @property
    def verbose(self):
        return self.__opts.verbose


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdCognitoDevices:{credentials_name:%s, find:%s, update:%s, delete:%s, " \
               "tag:%s, invoice:%s, memberships:%s, indent:%s, verbose:%s}" % \
               (self.credentials_name, self.find, self.update, self.delete,
                self.tag, self.invoice_number, self.memberships, self.indent, self.verbose)



"""
Created on 29 Jun 2021

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)
"""

import optparse

from scs_analysis import version
from scs_core.aws.monitor.alert.alert_status_intercourse import AlertStatusFindRequest


# --------------------------------------------------------------------------------------------------------------------

class CmdAlertStatus(object):
    """unix command line handler"""

    __CAUSES = {'L': '<L', 'U': '>U', 'N': 'NV', 'OK': 'OK'}

    # --------------------------------------------------------------------------------------------------------------------

    def __init__(self):
        """
        Constructor
        """
        causes = ' | '.join(self.__CAUSES)

        self.__parser = optparse.OptionParser(usage="%prog [-c CREDENTIALS] { -F { -l | -d [-a CAUSE] } | -D } "
                                                    "[-i INDENT] [-v] ID", version=version())

        # identity...
        self.__parser.add_option("--credentials", "-c", type="string", action="store", dest="credentials_name",
                                 help="the stored credentials to be presented")

        # operations...
        self.__parser.add_option("--find", "-F", action="store_true", dest="find", default=False,
                                 help="find alert status reports")

        self.__parser.add_option("--delete", "-D", action="store_true", dest="delete", default=False,
                                 help="delete the alert status history")

        # filters...
        self.__parser.add_option("--latest", "-l", action="store_true", dest="latest", default=False,
                                 help="find latest status report only")

        self.__parser.add_option("--history", "-d", action="store_true", dest="history", default=False,
                                 help="find history of status reports")

        self.__parser.add_option("--cause", "-a", type="string", action="store", dest="cause",
                                 help="filter history by cause { %s }" % causes)

        # output...
        self.__parser.add_option("--indent", "-i", type="int", action="store", dest="indent",
                                 help="pretty-print the output with INDENT")

        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    def is_valid(self):
        if self.id is None:
            return False

        if self.find == self.delete:
            return False

        if self.find and self.latest == self.history:
            return False

        if not self.find and (self.latest or self.history):
            return False

        if self.latest and self.__opts.cause is not None:
            return False

        if self.__opts.cause is not None and self.__opts.cause not in self.__CAUSES:
            return False

        if not self.__args or len(self.__args) != 1:
            return False

        return True


    def response_mode(self):
        return AlertStatusFindRequest.Mode.LATEST if self.latest else AlertStatusFindRequest.Mode.HISTORY


    # ----------------------------------------------------------------------------------------------------------------
    # properties: identity...

    @property
    def credentials_name(self):
        return self.__opts.credentials_name


    @property
    def id(self):
        return self.__args[0] if self.__args else None


    # ----------------------------------------------------------------------------------------------------------------
    # properties: operations...

    @property
    def find(self):
        return self.__opts.find


    @property
    def delete(self):
        return self.__opts.delete


    # ----------------------------------------------------------------------------------------------------------------
    # properties: filters...

    @property
    def latest(self):
        return self.__opts.latest


    @property
    def history(self):
        return self.__opts.history


    @property
    def cause(self):
        return None if self.__opts.cause is None else self.__CAUSES[self.__opts.cause]


    # ----------------------------------------------------------------------------------------------------------------
    # properties: output...

    @property
    def indent(self):
        return self.__opts.indent


    @property
    def verbose(self):
        return self.__opts.verbose


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdAlertStatus:{credentials_name:%s, id:%s, find:%s, delete:%s, latest:%s, history:%s, cause:%s, " \
               "indent:%s, verbose:%s}" % \
               (self.credentials_name, self.id, self.find, self.delete, self.latest, self.history, self.__opts.cause,
                self.indent, self.verbose)



#!/usr/bin/env python3

"""
Created on 10 Mar 2020

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis

DESCRIPTION
The csv_segmentor utility is used to segment the input stream of JSON documents into CSV files whose rows
have contiguous datetime values.

Contiguity is defined by the --max-interval flag. If the time interval between a document and the previous document is
greater than this interval, then the current CSV file is closed, and a new file is opened. File names (and
subdirectories) as specified by the --file-prefix flag. The datetime of the first row of CSV file is appended to the
prefix.

The input documents must contain a field carrying an ISO 8601 datetime. If the field in a given document is empty or
malformed, the document is ignored. If the field is not present in any document, the csv_segmentor utility
terminates.

The csv_segmentor utility generates a report giving the specifications of each contiguous block. If no file prefix
is given, then the CSV files are not generated, but the report is still produced.

SYNOPSIS
csv_segmentor.py -m { [DD-]HH:MM[:SS] | :SS } [-i ISO] [-f FILE_PREFIX] [-v]

EXAMPLES
csv_reader.py -v scs-bgx-508-gases-2020-Q1.csv | \
csv_segmentor.py -m 06:00 -f segments/scs-bgx-508-gases-2020-Q1 -v | \
csv_writer.py -v segments/scs-bgx-508-gases-2020-Q1-report.csv

FILES
Output file names are of the form:
FILE-PREFIX_BLOCK_START_DATETIME.csv

DOCUMENT EXAMPLE - OUTPUT
{"start": "2019-01-01T00:00:01Z", "end": "2019-01-04T10:04:51Z", "prev-interval": "",
"max-interval": "00-00:00:11", "count": 29550}
{"start": "2019-01-04T10:29:21Z", "end": "2019-01-04T10:37:41Z", "prev-interval": "00-00:24:30",
"max-interval": "00-00:00:10", "count": 51}
{"start": "2019-01-04T11:35:49Z", "end": "2019-01-04T11:41:19Z", "prev-interval": "00-00:58:07",
"max-interval": "00-00:00:10", "count": 34}

SEE ALSO
scs_analysis/csv_collator
"""

import sys

from scs_analysis.cmd.cmd_csv_segmentor import CmdCSVSegmentor
from scs_analysis.handler.csv_segmentor import CSVSegmentor

from scs_core.data.datetime import LocalizedDatetime
from scs_core.data.json import JSONify
from scs_core.data.path_dict import PathDict


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    segmentor = None

    document_count = 0
    processed_count = 0

    # ----------------------------------------------------------------------------------------------------------------
    # cmd...

    cmd = CmdCSVSegmentor()

    if not cmd.is_valid_interval():
        print("csv_segmentor: invalid format for max interval.", file=sys.stderr)
        exit(2)

    if not cmd.is_valid():
        cmd.print_help(sys.stderr)
        exit(2)

    if cmd.verbose:
        print("csv_segmentor: %s" % cmd, file=sys.stderr)

    try:
        # ------------------------------------------------------------------------------------------------------------
        # resources...

        segmentor = CSVSegmentor(cmd.max_interval, cmd.file_prefix)

        if cmd.verbose:
            print("csv_segmentor: %s" % segmentor, file=sys.stderr)
            sys.stderr.flush()


        # ------------------------------------------------------------------------------------------------------------
        # run...

        # receive...
        for line in sys.stdin:
            datum = PathDict.construct_from_jstr(line)

            if datum is None:
                break

            document_count += 1

            if cmd.iso not in datum.paths():
                raise KeyError(cmd.iso)

            rec = LocalizedDatetime.construct_from_iso8601(datum.node(cmd.iso))

            if rec is None:
                continue

            try:
                segmentor.collate(rec, line)
            except FileNotFoundError as ex:
                print("csv_segmentor: %s" % ex, file=sys.stderr)
                exit(1)

            processed_count += 1

        # report...
        for block in segmentor.blocks:
            print(JSONify.dumps(block))
            sys.stdout.flush()


    # ----------------------------------------------------------------------------------------------------------------
    # end...

    except KeyboardInterrupt:
        print(file=sys.stderr)

    except KeyError as ex:
        print("csv_segmentor: %s" % repr(ex), file=sys.stderr)
        exit(1)

    finally:
        if segmentor is not None:
            segmentor.close()

        if cmd.verbose:
            print("csv_segmentor: documents: %d processed: %d segments: %d" %
                  (document_count, processed_count, len(segmentor)), file=sys.stderr)



"""
Created on 20 Apr 2021

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)
"""

import optparse

from scs_analysis import version
from scs_core.aws.manager.configuration.configuration_intercourse import ConfigurationRequest


# --------------------------------------------------------------------------------------------------------------------

class CmdConfigurationMonitor(object):
    """unix command line handler"""

    def __init__(self):
        """
        Constructor
        """
        self.__parser = optparse.OptionParser(usage="%prog [-c CREDENTIALS] [-t TAG [-x]] { -l | -f | -d | -o } "
                                                    "[-i INDENT] [-v]", version=version())

        # identity...
        self.__parser.add_option("--credentials", "-c", type="string", action="store", dest="credentials_name",
                                 help="the stored credentials to be presented")

        # filters...
        self.__parser.add_option("--tag-filter", "-t", type="string", action="store", dest="tag_filter",
                                 help="the (partial) tag of the device(s)")

        self.__parser.add_option("--exactly", "-x", action="store_true", dest="exact_match", default=False,
                                 help="exact match for tag")

        # mode...
        self.__parser.add_option("--latest", "-l", action="store_true", dest="latest", default=False,
                                 help="report latest configuration for each device")

        self.__parser.add_option("--full-history", "-f", action="store_true", dest="history", default=False,
                                 help="report full configuration history")

        self.__parser.add_option("--diff-history", "-d", action="store_true", dest="diff", default=False,
                                 help="report configuration differences")

        self.__parser.add_option("--tags-only", "-o", action="store_true", dest="tags_only", default=False,
                                 help="report device tags only")

        # output...
        self.__parser.add_option("--indent", "-i", type="int", action="store", dest="indent",
                                 help="pretty-print the output with INDENT")

        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    def is_valid(self):
        count = 0

        if self.latest:
            count += 1

        if self.history:
            count += 1

        if self.diff:
            count += 1

        if self.tags_only:
            count += 1

        if count != 1:
            return False

        if self.exact_match and self.tag_filter is None:
            return False

        if self.__args:
            return False

        return True


    def response_mode(self):
        if self.latest:
            return ConfigurationRequest.Mode.LATEST

        if self.history:
            return ConfigurationRequest.Mode.HISTORY

        if self.diff:
            return ConfigurationRequest.Mode.DIFF

        if self.tags_only:
            return ConfigurationRequest.Mode.TAGS_ONLY

        return None


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def credentials_name(self):
        return self.__opts.credentials_name


    @property
    def tag_filter(self):
        return self.__opts.tag_filter


    @property
    def exact_match(self):
        return self.__opts.exact_match


    @property
    def latest(self):
        return self.__opts.latest


    @property
    def history(self):
        return self.__opts.history


    @property
    def diff(self):
        return self.__opts.diff


    @property
    def tags_only(self):
        return self.__opts.tags_only


    @property
    def indent(self):
        return self.__opts.indent


    @property
    def verbose(self):
        return self.__opts.verbose


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdConfigurationMonitor:{credentials_name:%s, tag_filter:%s, exact_match:%s, latest:%s, history:%s, " \
               "diff:%s, tags_only:%s, indent:%s, verbose:%s}" % \
               (self.credentials_name, self.tag_filter, self.exact_match, self.latest, self.history,
                self.diff, self.tags_only, self.indent, self.verbose)



"""
Created on 2 Mar 2019

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis
"""

import re

from collections import OrderedDict

from scs_core.csv.csv_writer import CSVWriter

from scs_core.data.datum import Datum
from scs_core.data.json import JSONable


# --------------------------------------------------------------------------------------------------------------------

class CSVCollator(object):
    """
    classdocs
    """

    # ----------------------------------------------------------------------------------------------------------------

    @classmethod
    def construct(cls, dataset_lower, dataset_upper, delta, file_prefix):
        bins = []

        bin_lower = dataset_lower
        numeric_format = Datum.format(dataset_upper, True)

        while bin_lower < dataset_upper:
            bin_upper = bin_lower + delta
            bins.append(CSVCollatorBin.construct(bin_lower, bin_upper, file_prefix, numeric_format))

            bin_lower = bin_upper

        return CSVCollator(dataset_lower, delta, bins)


    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, dataset_lower, delta, bins):
        """
        Constructor
        """
        self.__dataset_lower = dataset_lower            # float
        self.__delta = delta                            # float
        self.__bins = bins                              # array of CSVCollatorBin

        self.__max_bin_index = len(bins) - 1            # int


    # ----------------------------------------------------------------------------------------------------------------

    def collate(self, value, jstr):
        index = int((value - self.__dataset_lower) // self.__delta)

        if index < 0 or index > self.__max_bin_index:
            return False

        self.__bins[index].append(jstr)

        return True


    def close(self):
        for b in self.__bins:
            b.close()


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def dataset_lower(self):
        return self.__dataset_lower


    @property
    def delta(self):
        return self.__delta


    @property
    def bins(self):
        return self.__bins


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "CSVCollator:{dataset_lower:%s, delta:%s, bins:%s}" % \
               (self.dataset_lower, self.delta, self.bins)


# --------------------------------------------------------------------------------------------------------------------

class CSVCollatorBin(JSONable):
    """
    classdocs
    """

    # ----------------------------------------------------------------------------------------------------------------
    # e.g. scs-bgx-431-ref-particulates-N2-climate-2019_15min_clipped_085p0_090p0.csv

    @classmethod
    def construct(cls, lower, upper, file_prefix, numeric_format):
        if file_prefix is None:
            return cls(lower, upper, None)

        file_name = file_prefix + cls.__infix(numeric_format, lower) + cls.__infix(numeric_format, upper) + '.csv'
        writer = CSVWriter(file_name)

        return cls(lower, upper, writer)


    @classmethod
    def __infix(cls, numeric_format, bound):
        return str.replace("_" + numeric_format % bound, ".", "p")


    @staticmethod
    def parse(filename):
        match = re.match(r'.*_(\d+)p(\d+)_(\d+)p(\d+)\.csv$', filename)

        if match is None:
            raise ValueError(filename)

        fields = match.groups()

        low = float('.'.join(fields[0:2]))
        high = float('.'.join(fields[2:4]))

        return low, high


    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, lower, upper, writer):
        """
        Constructor
        """
        self.__lower = lower                            # float
        self.__upper = upper                            # float
        self.__writer = writer                          # CSVWriter

        self.__count = 0                                # int


    # ----------------------------------------------------------------------------------------------------------------

    def append(self, datum):
        if self.__writer:
            self.__writer.write(datum)

        self.__count += 1


    def close(self):
        if self.__writer:
            self.__writer.close()


    # ----------------------------------------------------------------------------------------------------------------

    def as_json(self):
        jdict = OrderedDict()

        jdict['lower'] = self.lower
        jdict['upper'] = self.upper
        jdict['count'] = self.count

        return jdict


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def lower(self):
        return self.__lower


    @property
    def upper(self):
        return self.__upper


    @property
    def count(self):
        return self.__count


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "CSVCollatorBin:{lower:%s, upper:%s, count:%s, writer:%s}" % \
               (self.lower, self.upper, self.count, self.__writer)



#!/usr/bin/env python3

"""
Created on 18 Nov 2016

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis

DESCRIPTION
The aws_topic_publisher utility is used to prepare data for publication by the aws_mqtt_client script. The
aws_topic_publisher acts by taking data from stdin, wrapping it in a JSON document whose only field has the name of
the given topic, and presenting the result on stdout.

Note that the aws_topic_publisher in scs_analysis necessarily works differently to the aws_topic_publisher in scs_dev.
This is because scs_dev version has access to the device's project specification, and therefore can find the topic path
automatically. For the scs_analysis version, the full topic path should be given explicitly.

SYNOPSIS
aws_topic_publisher.py -t TOPIC [-v]

EXAMPLES
aws_topic_publisher.py -t /users/southcoastscience-dev/test/json

SEE ALSO
scs_analysis/aws_mqtt_client
"""

import json
import sys

from scs_analysis.cmd.cmd_aws_topic_publisher import CmdAWSTopicPublisher

from scs_core.data.json import JSONify
from scs_core.data.publication import Publication


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    # ----------------------------------------------------------------------------------------------------------------
    # cmd...

    cmd = CmdAWSTopicPublisher()

    if not cmd.is_valid():
        cmd.print_help(sys.stderr)
        exit(2)

    if cmd.verbose:
        print("aws_topic_publisher: %s" % cmd, file=sys.stderr)

    try:
        # ------------------------------------------------------------------------------------------------------------
        # run...

        for line in sys.stdin:
            try:
                jdict = json.loads(line)
            except ValueError:
                continue

            publication = Publication(cmd.topic, jdict)

            print(JSONify.dumps(publication))
            sys.stdout.flush()


    # ----------------------------------------------------------------------------------------------------------------
    # end...

    except KeyboardInterrupt:
        print(file=sys.stderr)



#!/usr/bin/env python3

"""
Created on 17 Jul 2023

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis

DESCRIPTION
The sample_gas_density utility is used to

SYNOPSIS
sample_gas_density.py [-p PRECISION] [-v] PATH

EXAMPLES
aws_topic_history.py -vl south-coast-science-demo/brighton-urban/loc/1/gases | sample_gas_density.py -v exg.val

DOCUMENT EXAMPLE - INPUT
{... ,
"exg": {"val": {"NO2": {"cnc": 15.2}, "NO": {"cnc": 11.9}, "SO2": {"cnc": 1.1}},
"src": "uE.1"}}

DOCUMENT EXAMPLE - OUTPUT
{... ,
"exg": {"val": {"NO2": {"cnc": 15.2, "dns": 29.1}, "NO": {"cnc": 11.9, "dns": 15.9}, "SO2": {"cnc": 1.1, "dns": 2.9}},
"src": "uE.1"}}
"""

import sys

from scs_analysis.cmd.cmd_sample_gas_density import CmdSampleGasDensity

from scs_core.data.json import JSONify
from scs_core.data.path_dict import PathDict

from scs_core.gas.gas import Gas

from scs_core.sys.logging import Logging


# --------------------------------------------------------------------------------------------------------------------

def gas_name(path):
    pieces = path.split('.')
    return pieces[len(pieces) - 2]


def is_concentration(path):
    pieces = path.split('.')
    return pieces[len(pieces) - 1] == 'cnc'


def density_path(path):
    pieces = path.split('.')
    pieces[len(pieces) - 1] = 'dns'

    return '.'.join(pieces)


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    document_count = 0
    processed_count = 0

    # ----------------------------------------------------------------------------------------------------------------
    # cmd...

    cmd = CmdSampleGasDensity()

    if not cmd.is_valid():
        cmd.print_help(sys.stderr)
        exit(2)

    Logging.config('sample_gas_density', verbose=cmd.verbose)
    logger = Logging.getLogger()

    logger.info(cmd)


    try:
        # ------------------------------------------------------------------------------------------------------------
        # run...

        max_datum = None

        for line in sys.stdin:
            datum = PathDict.construct_from_jstr(line)

            if datum is None:
                continue

            document_count += 1

            value_paths = datum.paths(sub_path=cmd.path)

            for value_path in value_paths:
                if not is_concentration(value_path):
                    continue

                try:
                    concentration = float(datum.node(sub_path=value_path))
                except (TypeError, ValueError):
                    logger.error("invalid value for %s concentration in %s" % (gas_name(value_path), line.strip()))
                    exit(1)

                try:
                    density = Gas.density_stp(gas_name(value_path), concentration)
                except ValueError:
                    logger.error("unrecognised gas '%s' in %s" % (gas_name(value_path), line.strip()))
                    exit(1)

                datum.append(density_path(value_path), round(density, cmd.precision))

            # report...
            print(JSONify.dumps(datum))

            processed_count += 1


    # ----------------------------------------------------------------------------------------------------------------
    # end...

    except KeyboardInterrupt:
        print(file=sys.stderr)

    except KeyError as ex:
        logger.error(repr(ex))
        exit(1)

    finally:
        logger.info("documents: %d processed: %d" % (document_count, processed_count))



"""
Created on 4 Sep 2019

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis
"""

import optparse

from scs_analysis import version


# --------------------------------------------------------------------------------------------------------------------

class CmdSampleNullify(object):
    """unix command line handler"""

    def __init__(self):
        """
        Constructor
        """
        self.__parser = optparse.OptionParser(usage="%prog -t TARGET_PATH -s SOURCE_PATH [-l LOWER] [-u UPPER]"
                                                    " [-v]", version=version())

        # input...
        self.__parser.add_option("--target", "-t", type="string", action="store", dest="target",
                                 help="field to be nullified")

        self.__parser.add_option("--source", "-s", type="string", action="store", dest="source",
                                 help="field providing the test value")

        # operation...
        self.__parser.add_option("--lower", "-l", type="float", action="store", dest="lower",
                                 help="lower bound")

        self.__parser.add_option("--upper", "-u", type="float", action="store", dest="upper",
                                 help="upper bound")

        # output...
        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    def is_valid(self):
        if self.target is None or self.source is None:
            return False

        if self.lower is not None and self.upper is not None and self.upper <= self.lower:
            return False

        if self.__args:
            return False

        return True


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def target(self):
        return self.__opts.target


    @property
    def source(self):
        return self.__opts.source


    @property
    def lower(self):
        return self.__opts.lower


    @property
    def upper(self):
        return self.__opts.upper


    @property
    def verbose(self):
        return self.__opts.verbose


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdSampleNullify:{target:%s, source:%s, lower:%s, upper:%s, verbose:%s}" % \
               (self.target, self.source, self.lower, self.upper, self.verbose)



#!/usr/bin/env python3

"""
Created on 23 Oct 2020

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis

DESCRIPTION
The sample_slope utility it used to find the slope for a given field in a sequence of JSON documents. Slope is
defined as delta value / delta time (in seconds).

The output document replaced the input field with field.cur (the current value) and field.slope. For the initial
input document, the corresponding output document's field.slope value is null. A --tally flag specifies the number
of rolling samples in the regression.

The --max-interval flag is intended to prevent slope analyses covering long periods such as device downtime.

If the --exclude-incomplete flag is set, then any data proir to the required number of tallies is not output. Where
a chain of slope analyses are being performed, the flag should only be used on the last (highest tally) pass.

SYNOPSIS
sample_slope.py -n NAME [-i ISO] [-t TALLY] [-m [DD-]HH:MM[:SS]] [-x] [-p PRECISION] [-v] PATH

EXAMPLES
bruno:scs-opc-164 bruno$ csv_reader.py -v scs-opc-164-meteo-pmx-2022-15min.csv | \
sample_slope.py -v -n 15min -t2 -m 00:15:00 -x meteo.val.hmd | \
csv_writer.py -v scs-opc-164-meteo-pmx-2022-15min-slope.csv

DOCUMENT EXAMPLE - INPUT
{"rec": "2022-09-13T13:30:00Z", "val": {"hmd": 72.5, "tmp": 22.2,
"bar": {"pA": 100.964}}, "ver": 1.0, "tag": "scs-opc-125"}

DOCUMENT EXAMPLE - OUTPUT
{"rec": "2022-09-13T13:30:00Z", "val": {"hmd": {"cur": 72.5, "slp15min": -0.000111, "slp30min": 0.0005},
"tmp": 22.2, "bar": {"pA": 100.964}}, "ver": 1.0, "tag": "scs-opc-125"}
"""

import sys

from scs_analysis.cmd.cmd_sample_slope import CmdSampleSlope

from scs_core.data.datetime import LocalizedDatetime
from scs_core.data.datum import Datum
from scs_core.data.json import JSONify
from scs_core.data.linear_regression import LinearRegression
from scs_core.data.path_dict import PathDict

from scs_core.sys.logging import Logging


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    document_count = 0
    processed_count = 0

    # ----------------------------------------------------------------------------------------------------------------
    # cmd...

    cmd = CmdSampleSlope()

    if not cmd.is_valid():
        cmd.print_help(sys.stderr)
        exit(2)

    Logging.config('sample_slope', verbose=cmd.verbose)
    logger = Logging.getLogger()

    if not cmd.is_valid_interval():
        logger.error("invalid format for max interval.")
        exit(2)

    logger.info(cmd)

    try:
        # ------------------------------------------------------------------------------------------------------------
        # resources...

        regression = LinearRegression(tally=cmd.tally)

        source_path = cmd.path

        # ------------------------------------------------------------------------------------------------------------
        # run...

        prev_rec = None
        prev_value = None

        for line in sys.stdin:
            jstr = line.strip()
            datum = PathDict.construct_from_jstr(jstr)

            if datum is None:
                continue

            document_count += 1

            paths = datum.paths()

            # rec...
            if cmd.iso not in paths:
                logger.error("field %s not present." % cmd.iso)
                exit(1)

            rec_node = datum.node(cmd.iso)
            rec = LocalizedDatetime.construct_from_iso8601(rec_node)

            if rec is None:
                logger.error("invalid ISO 8601 value '%s' in %s." % (rec_node, jstr))
                exit(1)

            # exclude long intervals...
            if cmd.max_interval and prev_rec is not None and rec - prev_rec > cmd.max_interval:
                logger.info("regression reset on %s" % jstr)
                regression.reset()

            # value...
            if source_path not in paths:
                if source_path + '.cur' in paths:
                    source_path += '.cur'               # we are extending an existing slope analysis
                else:
                    logger.error("field %s not present in %s." % (cmd.path, jstr))
                    exit(1)

            value_node = datum.node(source_path)
            value = Datum.float(value_node)

            if value is None:
                logger.error("invalid numeric value '%s' in %s." % (value_node, jstr))
                exit(1)

            # slope...
            regression.append(rec, value)

            if regression.has_regression():
                slope = regression.slope(precision=cmd.precision)

            else:
                slope = None

            # target...
            target = PathDict()

            for path in datum.paths():
                if path == source_path:
                    target.append('.'.join((cmd.path, 'cur')), value)
                    target.append('.'.join((cmd.path, 'slp' + cmd.name)), slope)
                    continue

                target.copy(datum, path)

            prev_rec = rec
            prev_value = value

            if cmd.exclude_incomplete and (slope is None or document_count < cmd.tally):
                continue

            print(JSONify.dumps(target))
            sys.stdout.flush()

            processed_count += 1


    # ----------------------------------------------------------------------------------------------------------------
    # end...

    except KeyboardInterrupt:
        print(file=sys.stderr)

    finally:
        logger.info("documents: %d processed: %d" % (document_count, processed_count))



"""
Created on 15 Jul 2017

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis
"""

import matplotlib

from abc import ABC, abstractmethod
from matplotlib import pyplot as plt


# --------------------------------------------------------------------------------------------------------------------

class Chart(ABC):
    """
    classdocs
    """

    @staticmethod
    def backend():
        return matplotlib.rcParams['backend']


    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, title, batch_mode):
        """
        Constructor
        """
        self.__title = title                        # string
        self.__batch_mode = batch_mode              # bool

        self._closed = False                        # bool


    # ----------------------------------------------------------------------------------------------------------------

    @abstractmethod
    def plot(self, pathdict):
        pass


    @abstractmethod
    def render(self, block=True):
        pass


    def pause(self):
        if not self.__batch_mode:
            plt.pause(0.1)


    def close(self, _event):
        self._closed = True


    # ----------------------------------------------------------------------------------------------------------------

    def title(self, content):
        return content if self.__title is None else ' '.join((self.__title, content))


    @property
    def batch_mode(self):
        return self.__batch_mode


    @property
    def closed(self):
        return self._closed



"""
Created on 20 Feb 2017

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis
"""

import optparse

from scs_analysis import version


# --------------------------------------------------------------------------------------------------------------------

class CmdLocalizedDatetime(object):
    """unix command line handler"""

    def __init__(self):
        """
        Constructor
        """
        self.__parser = optparse.OptionParser(usage="%prog { [-o HOURS] [-m MINUTES] [-s SECONDS] [-t TIMEZONE_NAME] | "
                                                    "-z }", version=version())

        # fields...
        self.__parser.add_option("--hours", "-o", type="int", default=0, action="store", dest="hours",
                                 help="offset from now in hours")

        self.__parser.add_option("--minutes", "-m", type="int", default=0, action="store", dest="minutes",
                                 help="offset from now in minutes")

        self.__parser.add_option("--seconds", "-s", type="int", default=0, action="store", dest="seconds",
                                 help="offset from now in seconds")

        self.__parser.add_option("--timezone", "-t", type="string", action="store", dest="zone",
                                 help="present the time in the given zone")

        # helper...
        self.__parser.add_option("--zones", "-z", action="store_true", dest="list", default=False,
                                 help="list the available timezone names to stderr")

        # output...
        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    def is_valid(self):
        if self.list and (self.hours or self.minutes or self.seconds or self.zone):
            return False

        if self.__args:
            return False

        return True


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def hours(self):
        return self.__opts.hours


    @property
    def minutes(self):
        return self.__opts.minutes


    @property
    def seconds(self):
        return self.__opts.seconds


    @property
    def zone(self):
        return self.__opts.zone


    @property
    def list(self):
        return self.__opts.list


    @property
    def verbose(self):
        return self.__opts.verbose


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdLocalizedDatetime:{hours:%s, minutes:%s, seconds:%s, seconds:%s, seconds:%s, verbose:%s}" % \
               (self.hours, self.minutes, self.seconds, self.zone, self.list, self.verbose)



#!/usr/bin/env python3

"""
Created on 24 Jan 2022

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis

DESCRIPTION
The cognito_devices utility is used to create, update and retrieve AWS Cognito identities. This utility can only be used
by customer organisation administrators, SCS administrators and superusers.

If the --Create function is used, an email is sent to the new user. The verification link in the email must be
excercised using the cognito_email utility in order for the account to gain a CONFIRMED status.

SYNOPSIS
cognito_devices.py  [-c CREDENTIALS] { -F [{ -t TAG | -n INVOICE }] [-m] | -U TAG INVOICE | -D TAG } [-i INDENT] [-v]

EXAMPLES
cognito_devices.py -vi4 -c super -F -m

DOCUMENT EXAMPLE
{"username": "scs-bgx-401", "invoice": "INV-000123",
"created": "2023-06-23T10:32:52+01:00", "last-updated": "2023-06-23T10:32:52+01:00"}

SEE ALSO
scs_analysis/cognito_users
scs_analysis/cognito_user_credentials
scs_analysis/organisation_devices

RESOURCES
https://docs.aws.amazon.com/cognito/latest/developerguide/signing-up-users-in-your-app.html
https://docs.aws.amazon.com/cognito/latest/developerguide/user-pool-settings-policies.html
"""

import sys

from scs_analysis.cmd.cmd_cognito_devices import CmdCognitoDevices

from scs_core.aws.security.cognito_client_credentials import CognitoClientCredentials
from scs_core.aws.security.cognito_device import CognitoDeviceIdentity
from scs_core.aws.security.cognito_device_finder import CognitoDeviceFinder
from scs_core.aws.security.cognito_device_manager import CognitoDeviceManager
from scs_core.aws.security.cognito_login_manager import CognitoLoginManager
from scs_core.aws.security.cognito_membership import CognitoMembership
from scs_core.aws.security.organisation_manager import OrganisationManager

from scs_core.client.http_exception import HTTPException, HTTPConflictException

from scs_core.data.json import JSONify

from scs_core.sys.logging import Logging

from scs_host.sys.host import Host


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    logger = None
    report = None

    try:
        # ------------------------------------------------------------------------------------------------------------
        # cmd...

        cmd = CmdCognitoDevices()

        if not cmd.is_valid():
            cmd.print_help(sys.stderr)
            exit(2)

        Logging.config('cognito_devices', verbose=cmd.verbose)
        logger = Logging.getLogger()

        logger.info(cmd)


        # ------------------------------------------------------------------------------------------------------------
        # authentication...

        credentials = CognitoClientCredentials.load_for_user(Host, name=cmd.credentials_name)

        if not credentials:
            exit(1)

        gatekeeper = CognitoLoginManager()
        auth = gatekeeper.user_login(credentials)

        if not auth.is_ok():
            logger.error("login: %s." % auth.authentication_status.description)
            exit(1)


        # ------------------------------------------------------------------------------------------------------------
        # resources...

        device_manager = CognitoDeviceManager()
        org_manager = OrganisationManager()
        finder = CognitoDeviceFinder()


        # ------------------------------------------------------------------------------------------------------------
        # run...

        if cmd.find:
            if cmd.tag is None:
                report = sorted(finder.find_all(auth.id_token))

            else:
                report = sorted(finder.find_by_tag(auth.id_token, cmd.tag))

            if cmd.invoice_number is not None:
                report = [device for device in report if device.invoice_number == cmd.invoice_number]

            if cmd.memberships:
                org_devices = org_manager.find_devices(auth.id_token)
                report = CognitoMembership.merge(report, org_devices)

        if cmd.update:
            # find...
            device = finder.get_by_tag(auth.id_token, cmd.update_tag)

            if device is None:
                logger.error("no device found for tag: '%s'." % cmd.update)
                exit(1)

            # if not CognitoDeviceIdentity.is_valid_invoice_number(cmd.update_invoice):
            #     logger.error("'%s' is not a valid invoice number." % cmd.update_invoice)
            #     exit(2)

            # update...
            updated = CognitoDeviceIdentity(cmd.update_tag, None, cmd.update_invoice, None, None)
            report = device_manager.update(auth.id_token, updated)

        if cmd.delete:
            device_manager.delete(auth.id_token, cmd.delete)


    # ----------------------------------------------------------------------------------------------------------------
    # end...

        if report is not None:
            print(JSONify.dumps(report, indent=cmd.indent))

        if cmd.find:
            logger.info("found: %s" % len(report))

    except KeyboardInterrupt:
        print(file=sys.stderr)

    except HTTPConflictException as ex:
        logger.error("the tag '%s' is already in use." % report.tag)
        exit(1)

    except HTTPException as ex:
        logger.error(ex.error_report)
        exit(1)

    except Exception as ex:
        logger.error(ex.__class__.__name__)
        exit(1)



#!/usr/bin/env python3

"""
Created on 19 Sep 2022

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis

DESCRIPTION
The sample_sort utility is used to sort the JSON documents provided on stdin, according to one or more fields indicated
by the given paths. It is up to the user to ensure that each document contains a leaf node for each path, and that the
type of each node is compatible (i.e. all values for a given path are numeric or are strings).

WARNING: no support is given for localised datetimes. Therefore, for example, 2022-09-16T09:00:00Z is considered
not equal to 2022-09-16T10:00:00+01:00.

SYNOPSIS
Usage: sample_sort.py [-v] SORT_PATH_1 .. SORT_PATH_N

EXAMPLES
csv_reader.py -v ref-scs-bgx-508-20H1-slp16-vcal-err-clean-on-train-vE-exg.csv | \
sample_sort.py -v rec | \
csv_writer.py -v ref-scs-bgx-508-20H1-slp16-vcal-err-clean-on-train-vE-exg-sorted.csv
"""

import sys

from scs_analysis.cmd.cmd_sample_sort import CmdSampleSort

from scs_core.data.json import JSONify
from scs_core.data.path_dict import PathDict

from scs_core.sys.logging import Logging


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    document_count = 0

    # ----------------------------------------------------------------------------------------------------------------
    # cmd...

    cmd = CmdSampleSort()

    if not cmd.is_valid():
        cmd.print_help(sys.stderr)
        exit(2)

    Logging.config('sample_sort', verbose=cmd.verbose)
    logger = Logging.getLogger()

    logger.info(cmd)

    try:
        # ------------------------------------------------------------------------------------------------------------
        # run...

        data = []

        # ingest...
        for line in sys.stdin:
            datum = PathDict.construct_from_jstr(line, sort_paths=cmd.sort_paths)

            if datum is None:
                continue

            document_count += 1

            data.append(datum)

        # report...
        for datum in sorted(data):
            print(JSONify.dumps(datum))


    # ----------------------------------------------------------------------------------------------------------------
    # end...

    except KeyboardInterrupt:
        print(file=sys.stderr)

    except (KeyError, TypeError) as ex:
        logger.error(repr(ex))
        exit(1)

    finally:
        logger.info("documents: %d" % document_count)



#!/usr/bin/env python3

"""
Created on 24 Oct 2018

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis

DESCRIPTION
The sample_aggregate utility provides regression midpoints for data delivered on stdin, over specified units of
time, or over the entire dataset. It can perform this operation for one or many or all nodes of the input documents.

When each time checkpoint is encountered in the input stream, the midpoint values - together with min and max, if
requested -  are computed and reported. These values are marked with the datetime indicating the end of that period.
When the input stream is closed, any remaining values are reported and marked with the next checkpoint.

Checkpoints are specified in the form HH:MM:SS, in a format similar to that for Unix crontab:

** - all values
NN - exactly matching NN
/N - repeated every N

For example, **:/5:30 indicates 30 seconds past the minute, every 5 minutes, during every hour.

If no checkpoint is specified, then a single document is output - this is the aggregation of all input data.

Data sources are specified as a path into the input JSON document in the same format as the node command. Any number of
paths can be specified, including none (process all paths). If a path to an internal node in the JSON document is
specified, then all of the leaf-node descendants of that node will be processed.

Note that the leaf-node paths to be processed are obtained from the paths provided on the command line, and the
actual paths found in the first JSON document. Paths that do not exist in the first document are ignored.

The input JSON document must contain a field labelled 'rec', providing an ISO 8601 localised datetime. If this field
is not present then the document is skipped. Note that the timezone of the output rec datetimes is the same as the
input rec values. Rows with successive duplicate rec values are ignored.

Leaf node values may be numeric or strings. Numeric values are processed according to a simple linear regression.
String values are processed using a simple categorical regression.

If the input document does not contain a specified path - or if the value is null - then the value is ignored.

If a checkpoint is specified and the --exclude-remainder flag is used, then all of the input documents after the last
complete checkpoint period are ignored. Where the checkpoint is specified, a --rule flag is available. If used,
individual aggregates are rejected if less than 75% of the expected data points are present. In this case, a timedelta
must be supplied, indicating indicating the expected interval between the input samples. The interval may be found
using the aws_topic_history utility.

WARNING: The The sample_aggregate utility uses the first input document to determine the data type for the regressions.
If csv_reader is being used to supply data, then the csv_reader's --nullify flag should be used - this will prevent
numeric fields being incorrectly identified as strings.

SYNOPSIS
sample_aggregate.py [-c HH:MM:SS [-x] [-r { [DD-]HH:MM[:SS]] | :SS }]] [-m] [-i ISO] [-v] [PATH_1..PATH_N]

EXAMPLES
csv_reader.py -n gases.csv | sample_aggregate.py -v -r :10 -c **:/15:00

aws_topic_history.py -v -c super -p 00:00:00 -s 2023-12-01T00:00:00Z -e 2024-01-01T00:00:00Z \
south-coast-science-production/reference/loc/531/particulates | node.py tag rec ver src val.sfr val.sht exg | \
sample_aggregate.py | csv_writer.py -e 531-particulates-2023-12.csv
"""

import sys

from scs_analysis.cmd.cmd_sample_aggregate import CmdSampleAggregate

from scs_core.data.aggregate import Aggregate
from scs_core.data.checkpoint_generator import CheckpointGenerator
from scs_core.data.datetime import LocalizedDatetime
from scs_core.data.json import JSONify
from scs_core.data.path_dict import PathDict

from scs_core.sys.logging import Logging


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':
    rec = None
    prev_checkpoint = None
    checkpoint = None
    prev_rec = None

    document_count = 0
    processed_count = 0
    output_count = 0
    rejected_count = 0

    # ----------------------------------------------------------------------------------------------------------------
    # cmd...

    cmd = CmdSampleAggregate()

    if not cmd.is_valid():
        cmd.print_help(sys.stderr)
        exit(2)

    Logging.config('sample_aggregate', verbose=cmd.verbose)
    logger = Logging.getLogger()

    logger.info(cmd)


    # ----------------------------------------------------------------------------------------------------------------
    # validation...

    if cmd.checkpoint and not CheckpointGenerator.is_valid(cmd.checkpoint):
        logger.error("the checkpoint specification %s is invalid." % cmd.checkpoint)
        exit(2)

    if not cmd.is_valid_interval():
        logger.error("invalid format for rule interval.")
        exit(2)


    try:
        # ------------------------------------------------------------------------------------------------------------
        # resources...

        generator = CheckpointGenerator.construct(cmd.checkpoint) if cmd.checkpoint else None

        aggregate = Aggregate(cmd.min_max, cmd.iso, cmd.nodes)
        logger.info(aggregate)


        # ------------------------------------------------------------------------------------------------------------
        # run...

        for line in sys.stdin:
            # sample...
            datum = PathDict.construct_from_jstr(line)

            if datum is None:
                continue

            document_count += 1

            try:
                rec_node = datum.node(cmd.iso)
            except KeyError:
                continue

            rec = LocalizedDatetime.construct_from_iso8601(rec_node)

            # set checkpoint...
            if generator and checkpoint is None:
                prev_checkpoint = generator.prev_localised_datetime(rec)
                checkpoint = generator.next_localised_datetime(rec)

            # report and reset...
            if checkpoint and rec > checkpoint:
                if cmd.ignore_rule() or aggregate.complies_with_rule(cmd.rule_interval, checkpoint - prev_checkpoint):
                    print(JSONify.dumps(aggregate.report(checkpoint)))
                    sys.stdout.flush()
                    output_count += 1

                else:
                    rejected_count += 1

                aggregate.reset()

                if generator:
                    prev_checkpoint = checkpoint
                    checkpoint = generator.enclosing_localised_datetime(rec)

            # duplicate recs?...
            if rec == prev_rec:
                logger.info("discarding duplicate: %s" % line.strip())
                continue

            # append sample...
            aggregate.append(rec, datum)

            prev_rec = rec
            processed_count += 1

        # report remainder...
        if generator is None:
            checkpoint = rec

        if aggregate.has_value() and not cmd.exclude_remainder:
            if cmd.ignore_rule() or aggregate.complies_with_rule(cmd.rule_interval, checkpoint - prev_checkpoint):
                print(JSONify.dumps(aggregate.report(checkpoint)))
                sys.stdout.flush()
                output_count += 1

            else:
                rejected_count += 1


    # ----------------------------------------------------------------------------------------------------------------
    # end...

    except KeyboardInterrupt:
        print(file=sys.stderr)

    finally:
        logger.info("documents: %d processed: %d output: %d rejected: %d" %
                    (document_count, processed_count, output_count, rejected_count))



"""
Created on 20 Feb 2017

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis
"""

import logging
import optparse

from scs_analysis import version

from scs_core.data.datetime import LocalizedDatetime
from scs_core.data.timedelta import Timedelta


# TODO: add backoff limit flag
# --------------------------------------------------------------------------------------------------------------------

class CmdAWSTopicHistory(object):
    """unix command line handler"""

    def __init__(self):
        """
        Constructor
        """
        self.__parser = optparse.OptionParser(usage="%prog [-c CREDENTIALS] { -l | -a LATEST_AT [-b BACK-OFF] | "
                                                    "-t { [[DD-]HH:]MM[:SS] | :SS } | -s START [-e END] } "
                                                    "{ -p HH:MM:SS [-m] [-x] | [-w] [-f] } [-r] [{ -v | -d }] TOPIC",
                                              version=version())

        # identity...
        self.__parser.add_option("--credentials", "-c", type="string", action="store", dest="credentials_name",
                                 help="the stored credentials to be presented")

        # functions...
        self.__parser.add_option("--latest", "-l", action="store_true", dest="latest", default=False,
                                 help="the most recent document")

        self.__parser.add_option("--latest-at", "-a", type="string", action="store", dest="latest_at",
                                 help="the most recent document before ISO 8601 datetime")

        self.__parser.add_option("--back-off", "-b", type="int", action="store", dest="back_off",
                                 help="maximum look-back period (seconds)")

        self.__parser.add_option("--timedelta", "-t", type="string", action="store", dest="timedelta",
                                 help="starting days / hours / minutes / seconds ago, and ending now")

        self.__parser.add_option("--start", "-s", type="string", action="store", dest="start",
                                 help="ISO 8601 datetime START")

        self.__parser.add_option("--end", "-e", type="string", action="store", dest="end",
                                 help="ISO 8601 datetime END")

        # aggregation...
        self.__parser.add_option("--checkpoint", "-p", type="string", action="store", dest="checkpoint",
                                 help="a time specification as **:/05:00")

        self.__parser.add_option("--min-max", "-m", action="store_true", dest="min_max", default=False,
                                 help="include min and max in addition to midpoint")

        self.__parser.add_option("--exclude-remainder", "-x", action="store_true", dest="exclude_remainder",
                                 help="ignore documents after the last complete period")

        # output...
        self.__parser.add_option("--wrapper", "-w", action="store_true", dest="include_wrapper", default=False,
                                 help="include storage wrapper")

        self.__parser.add_option("--fetch-last", "-f", action="store_true", dest="fetch_last", default=False,
                                 help="fetch the last available hour of data if the requested data is unavailable")

        self.__parser.add_option("--rec-only", "-r", action="store_true", dest="rec_only", default=False,
                                 help="retrieve only the rec field")

        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__parser.add_option("--debug", "-d", action="store_true", dest="debug", default=False,
                                 help="report narrative and request / response to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    def is_valid(self):
        if self.topic is None:
            return False

        if self.__opts.start is None and self.__opts.end is not None:
            return False

        count = 0

        if self.latest:
            count += 1

        if self.__opts.latest_at is not None:
            count += 1

        if self.__opts.timedelta is not None:
            count += 1

        if self.__opts.start is not None:
            count += 1

        if count != 1:
            return False

        if self.back_off is not None and not self.latest_at:
            return False

        if self.include_wrapper and self.checkpoint:
            return False

        if self.rec_only and self.fetch_last:
            return False

        if self.rec_only and self.min_max:
            return False

        if self.min_max and not self.checkpoint:
            return False

        if self.exclude_remainder and not self.checkpoint:
            return False

        if self.verbose and self.debug:
            return False

        if not self.__args or len(self.__args) != 1:
            return False

        return True


    def is_valid_latest_at(self):
        if self.__opts.latest_at is None:
            return True

        return self.latest_at is not None


    def is_valid_timedelta(self):
        if self.__opts.timedelta is None:
            return True

        return self.timedelta is not None


    def is_valid_start(self):
        if self.__opts.start is None:
            return True

        return self.start is not None


    def is_valid_end(self):
        if self.__opts.end is None:
            return True

        return self.end is not None


    def log_level(self):
        if self.debug:
            return logging.DEBUG

        if self.verbose:
            return logging.INFO

        return logging.ERROR


    # ----------------------------------------------------------------------------------------------------------------
    # properties: identity...

    @property
    def credentials_name(self):
        return self.__opts.credentials_name


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def latest(self):
        return self.__opts.latest


    @property
    def latest_at(self):
        return LocalizedDatetime.construct_from_iso8601(self.__opts.latest_at)


    @property
    def back_off(self):
        return self.__opts.back_off


    @property
    def timedelta(self):
        return Timedelta.construct_from_flag(self.__opts.timedelta)


    @property
    def start(self):
        return LocalizedDatetime.construct_from_iso8601(self.__opts.start)


    @property
    def end(self):
        return LocalizedDatetime.construct_from_iso8601(self.__opts.end)


    @property
    def fetch_last(self):
        return self.__opts.fetch_last


    @property
    def checkpoint(self):
        return self.__opts.checkpoint


    @property
    def include_wrapper(self):
        return self.__opts.include_wrapper


    @property
    def rec_only(self):
        return self.__opts.rec_only


    @property
    def min_max(self):
        return self.__opts.min_max


    @property
    def exclude_remainder(self):
        return self.__opts.exclude_remainder


    @property
    def verbose(self):
        return self.__opts.verbose


    @property
    def debug(self):
        return self.__opts.debug


    @property
    def topic(self):
        return self.__args[0] if len(self.__args) > 0 else None


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdAWSTopicHistory:{credentials_name:%s, latest:%s, latest_at:%s, latest_at:%s, timedelta:%s, " \
               "start:%s, end:%s, fetch_last:%s, checkpoint:%s, include_wrapper:%s, rec_only:%s, " \
               "min_max:%s, exclude_remainder:%s, verbose:%s, debug:%s, topic:%s}" % \
                    (self.credentials_name, self.latest, self.__opts.latest_at, self.back_off, self.__opts.timedelta,
                     self.start, self.end, self.fetch_last, self.checkpoint, self.include_wrapper, self.rec_only,
                     self.min_max, self.exclude_remainder, self.verbose, self.debug, self.topic)



#!/usr/bin/env python3

"""
Created on 17 Apr 2023

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis

DESCRIPTION
The device_controller utility is used to interact with a remote host, using the device's control topic.
A command / receipt message regime provides an interactive system over the messaging infrastructure.

An appropriate email address and password must have been stored using the cognito_credentials utility.

In the interactive mode, the aws_mqtt_control command-line interpreter supports history [UP] and [DOWN] keys.
The mode supports command completion with the [TAB] key and command listing with [TAB][TAB]. Exit from the interactive
mode with [CTRL-D].


In the batch message mode, each command should be separated by a semicolon (;). Each command is sent as a separate
message. Typically, message tokens are presented on the command line wrapped in double quote (") characters.

A maximum of 30 seconds is available for the device to respond to the published message. After this time, the
device_controller utility will terminate.

SYNOPSIS
device_controller.py [-c CREDENTIALS] -t DEVICE_TAG [{ [-w] [-i INDENT] | [-s] -m CMD_TOKENS }]] [-v]

EXAMPLES
device_controller.py -c super -t scs-be2-3 -s -m "vcal_baseline -i4; gas_baseline"

SEE ALSO
scs_analysis/cognito_user_credentials

BUGS
On macOS, interactive history features require full disk access for the Terminal app:

PermissionError: [Errno 1] Operation not permitted after macOS Catalina Update
https://stackoverflow.com/questions/58479686/permissionerror-errno-1-operation-not-permitted-after-macos-catalina-update
"""

import json
import os
import sys

from scs_analysis.cmd.cmd_device_controller import CmdDeviceController

from scs_core.aws.client.device_control_client import DeviceControlClient
from scs_core.aws.security.cognito_client_credentials import CognitoClientCredentials
from scs_core.aws.security.cognito_login_manager import CognitoLoginManager

from scs_core.client.http_exception import HTTPException, HTTPNotFoundException, HTTPGatewayTimeoutException, \
    HTTPServiceUnavailableException

from scs_core.data.json import AbstractPersistentJSONable, JSONify

from scs_core.sys.logging import Logging

from scs_host.comms.stdio import StdIO
from scs_host.sys.host import Host


# --------------------------------------------------------------------------------------------------------------------

EXIT_COMMANDS = ['reboot', 'restart', 'shutdown']


# --------------------------------------------------------------------------------------------------------------------

def print_output(command):
    if command.stderr:
        print(*command.stderr, sep='\n', file=sys.stderr)
        sys.stderr.flush()

    if command.stdout:
        print(*command.stdout, sep='\n')
        sys.stdout.flush()


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    history_filename = os.path.join(Host.scs_path(), AbstractPersistentJSONable.aws_dir(), 'device_controller.history')


    # ----------------------------------------------------------------------------------------------------------------
    # cmd...

    cmd = CmdDeviceController()

    if not cmd.is_valid():
        cmd.print_help(sys.stderr)
        exit(2)

    Logging.config('device_controller', verbose=cmd.verbose)
    logger = Logging.getLogger()

    logger.info(cmd)

    try:
        # ------------------------------------------------------------------------------------------------------------
        # authentication...

        credentials = CognitoClientCredentials.load_for_user(Host, name=cmd.credentials_name)

        if not credentials:
            exit(1)

        gatekeeper = CognitoLoginManager()
        auth = gatekeeper.user_login(credentials)

        if not auth.is_ok():
            logger.error("login: %s." % auth.authentication_status.description)
            exit(1)


        # ------------------------------------------------------------------------------------------------------------
        # resources...

        client = DeviceControlClient()


        # ------------------------------------------------------------------------------------------------------------
        # StdIO settings...

        if not cmd.message:
            response = client.interact(auth.id_token, cmd.device_tag, ['?'])

            if response.command.stderr:
                logger.error("%s device problem: %s." % (cmd.device_tag, response.command.stderr[0]))
                exit(1)

            commands = json.loads(response.command.stdout[0])
            vocabulary = [command + ' ' for command in commands]

            StdIO.set(vocabulary=vocabulary, history_filename=history_filename)


        # ------------------------------------------------------------------------------------------------------------
        # run...

        if cmd.message:
            return_code = 0
            for command in cmd.message.split(';'):
                logger.info(command)
                cmd_tokens = command.strip().split()
                response = client.interact(auth.id_token, cmd.device_tag, cmd_tokens)

                if cmd.std:
                    print_output(response.command)

                else:
                    report = response if cmd.wrapper else response.command
                    print(JSONify.dumps(report, indent=cmd.indent))

                if response.command.return_code != 0:
                    return_code = response.command.return_code

            exit(return_code)

        else:
            while True:
                line = StdIO.prompt(cmd.device_tag)

                if not line:
                    continue

                auth = gatekeeper.user_login(credentials)

                cmd_tokens = line.strip().split()
                response = client.interact(auth.id_token, cmd.device_tag, cmd_tokens)
                print_output(response.command)

                if cmd_tokens and cmd_tokens[0] in EXIT_COMMANDS:
                    exit(0)


    # ----------------------------------------------------------------------------------------------------------------
    # end...

    except KeyboardInterrupt:
        print(file=sys.stderr)

    except HTTPNotFoundException:
        logger.error("device '%s' not found." % cmd.device_tag)
        exit(1)

    except HTTPGatewayTimeoutException:
        logger.error("device '%s' is not available." % cmd.device_tag)
        exit(1)

    except HTTPServiceUnavailableException:
        logger.error("device '%s' is interacting with another controller." % cmd.device_tag)
        exit(1)

    except HTTPException as ex:
        logger.error(ex.error_report)
        exit(1)

    except Exception as ex:
        logger.error(ex.__class__.__name__)
        exit(1)

    finally:
        if not cmd.message:
            StdIO.save_history(history_filename)



"""
Created on 25 Apr 2017

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis
"""

import optparse

from scs_analysis import version

from scs_core.data.path_dict import PathDict


# --------------------------------------------------------------------------------------------------------------------

class CmdNode(object):
    """unix command line handler"""

    def __init__(self):
        """
        Constructor
        """
        self.__parser = optparse.OptionParser(usage="%prog [-x] [-a] [-s] [-f FILE] [-i INDENT] [-v] "
                                                    "[SUB_PATH_1 .. SUB_PATH_N]", version=version())

        # mode...
        self.__parser.add_option("--exclude", "-x", action="store_true", dest="exclude", default=False,
                                 help="include all sub-paths except the named one(s)")

        self.__parser.add_option("--array", "-a", action="store_true", dest="array", default=False,
                                 help="output the sequence of input JSON documents as array")

        self.__parser.add_option("--sequence", "-s", action="store_true", dest="sequence", default=False,
                                 help="output the contents of the input array node(s) as a sequence")

        self.__parser.add_option("--file", "-f", type="string", action="store", dest="filename",
                                 help="read from FILE instead of stdin")

        # output...
        self.__parser.add_option("--indent", "-i", type="int", action="store", dest="indent",
                                 help="pretty-print the output with INDENT")

        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    @classmethod
    def is_valid(cls):
        return True


    def includes(self, path):
        for sub_path in self.sub_paths:
            if PathDict.sub_path_includes_path(sub_path, path):
                return not self.exclude

        return self.exclude


    def has_sub_paths(self):
        return bool(self.__args)


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def exclude(self):
        return self.__opts.exclude


    @property
    def array(self):
        return self.__opts.array


    @property
    def sequence(self):
        return self.__opts.sequence


    @property
    def filename(self):
        return self.__opts.filename


    @property
    def indent(self):
        return self.__opts.indent


    @property
    def verbose(self):
        return self.__opts.verbose


    @property
    def sub_paths(self):
        return self.__args if self.__args else [None]           # if empty return only the root node


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdNode:{exclude:%s, array:%s, sequence:%s, filename:%s, indent:%s, verbose:%s, sub_paths:%s}" %  \
               (self.exclude, self.array, self.sequence, self.filename, self.indent, self.verbose, self.__args)



"""
Created on 16 Mar 2020

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis
"""

from scs_core.data.datetime import LocalizedDatetime
from scs_core.data.linear_regression import LinearRegression
from scs_core.data.path_dict import PathDict


# TODO: use named input_path and response_pth
# --------------------------------------------------------------------------------------------------------------------

class SampleRegression(object):
    """
    classdocs
    """

    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, path, tally, precision):
        """
        Constructor
        """
        self.__path = path
        self.__precision = precision

        self.__func = LinearRegression(tally=tally)


    # ----------------------------------------------------------------------------------------------------------------

    def datum(self, sample):
        if not sample.has_path(self.__path):
            return None

        try:
            rec_node = sample.node('rec')
        except KeyError:
            return None

        rec = LocalizedDatetime.construct_from_jdict(rec_node)
        value = sample.node(self.__path)

        self.__func.append(rec, value)

        if not self.__func.has_regression():
            return None

        slope, intercept = self.__func.line()

        if slope is None:
            return None

        target = PathDict()

        for path in sample.paths():
            if path == self.__path:
                target.append(self.__path + '.src', value)
                target.append(self.__path + '.slope', round(slope, self.__precision))
                target.append(self.__path + '.intercept', round(intercept, self.__precision))
            else:
                target.copy(sample, path)

        return target.node()


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "SampleRegression:{path:%s, func:%s}" % (self.__path, self.__func)



"""
Created on 10 Jan 2022

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)
"""

import optparse

from scs_analysis import version


# --------------------------------------------------------------------------------------------------------------------

class CmdOrganisations(object):
    """unix command line handler"""

    def __init__(self):
        """
        Constructor
        """
        self.__parser = optparse.OptionParser(usage="%prog [-c CREDENTIALS] { -F [{ -l LABEL | -d ID }] [-m] | "
                                                    "-C -l LABEL -n LONG_NAME -u URL -o OWNER_EMAIL "
                                                    "[-p PARENT_LABEL] | "
                                                    "-U LABEL [-l LABEL] [-n LONG_NAME] [-u URL] [-o OWNER_EMAIL] "
                                                    "[-p PARENT_LABEL] | "
                                                    "-D LABEL } "
                                                    "[-i INDENT] [-v]", version=version())

        # identity...
        self.__parser.add_option("--credentials", "-c", type="string", action="store", dest="credentials_name",
                                 help="the stored credentials to be presented")

        # operations...
        self.__parser.add_option("--Find", "-F", action="store_true", dest="find", default=False,
                                 help="find the organisations I belong to")

        self.__parser.add_option("--Create", "-C", action="store_true", dest="create", default=False,
                                 help="create an organisation")

        self.__parser.add_option("--Update", "-U", type="string", action="store", dest="update",
                                 help="update the organisation with the given LABEL")

        self.__parser.add_option("--Delete", "-D", type="string", action="store", dest="delete",
                                 help="delete the organisation")

        # fields...
        self.__parser.add_option("--label", "-l", type="string", action="store", dest="label",
                                 help="the organisation label")

        self.__parser.add_option("--id", "-d", type="int", action="store", dest="id",
                                 help="the organisation ID")

        self.__parser.add_option("--long-name", "-n", type="string", action="store", dest="long_name",
                                 help="the organisation long name")

        self.__parser.add_option("--url", "-u", type="string", action="store", dest="url",
                                 help="the organisation URL")

        self.__parser.add_option("--owner", "-o", type="string", action="store", dest="owner",
                                 help="the organisation owner email")

        self.__parser.add_option("--parent", "-p", type="string", action="store", dest="parent_label",
                                 help="the label of the parent organisation ('none' to remove)")

        # output...
        self.__parser.add_option("--memberships", "-m", action="store_true", dest="memberships", default=False,
                                 help="show organisation's children")

        self.__parser.add_option("--indent", "-i", type="int", action="store", dest="indent",
                                 help="pretty-print the output with INDENT")

        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    def is_valid(self):
        count = 0

        if self.find:
            count += 1

        if self.create:
            count += 1

        if self.update is not None:
            count += 1

        if self.delete is not None:
            count += 1

        if count != 1:
            return False

        if self.create and (self.label is None or self.long_name is None or self.url is None or self.owner is None):
            return False

        if self.label is not None and self.id is not None:
            return False

        if self.__args:
            return False

        return True


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def credentials_name(self):
        return self.__opts.credentials_name


    @property
    def find(self):
        return self.__opts.find


    @property
    def create(self):
        return self.__opts.create


    @property
    def update(self):
        return self.__opts.update


    @property
    def delete(self):
        return self.__opts.delete


    @property
    def label(self):
        return self.__opts.label


    @property
    def id(self):
        return self.__opts.id


    @property
    def long_name(self):
        return self.__opts.long_name


    @property
    def url(self):
        return self.__opts.url


    @property
    def owner(self):
        return self.__opts.owner


    @property
    def parent_label(self):
        return self.__opts.parent_label


    @property
    def memberships(self):
        return self.__opts.memberships


    @property
    def indent(self):
        return self.__opts.indent


    @property
    def verbose(self):
        return self.__opts.verbose


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdOrganisations:{credentials_name:%s, find:%s, create:%s, update:%s, delete:%s, " \
               "label:%s, id:%s, long_name:%s, url:%s, owner:%s, parent_label:%s, " \
               "memberships:%s, indent:%s, verbose:%s}" % \
               (self.credentials_name, self.find, self.create, self.update, self.delete,
                self.label, self.id, self.long_name, self.url, self.owner, self.parent_label,
                self.memberships, self.indent, self.verbose)



#!/usr/bin/env python3

"""
Created on 20 Apr 2021

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

DESCRIPTION
The monitor_auth utility is used to

A password must be specified when the key is created and is required when the key is accessed.

SYNOPSIS
monitor_auth.py [{ -s | -d }] [-v]

EXAMPLES
./monitor_auth.py -s

FILES
~/SCS/aws/monitor_auth.json

DOCUMENT EXAMPLE
{"email": "bruno.beloff@southcoastscience.com", "password": "XXX"}

SEE ALSO
scs_analysis/alert
scs_analysis/alert_status
scs_analysis/configuration_csv
scs_analysis/configuration_monitor
scs_analysis/configuration_monitor_status

RESOURCES
https://stackoverflow.com/questions/42568262/how-to-encrypt-text-with-a-password-in-python
"""

import sys

from scs_analysis.cmd.cmd_monitor_auth import CmdMonitorAuth

from scs_core.aws.client.monitor_auth import MonitorAuth
from scs_core.data.json import JSONify
from scs_core.sys.logging import Logging

from scs_host.sys.host import Host


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    auth = None

    try:
        # ------------------------------------------------------------------------------------------------------------
        # cmd...

        cmd = CmdMonitorAuth()

        if not cmd.is_valid():
            cmd.print_help(sys.stderr)
            exit(2)

        Logging.config('monitor_auth', verbose=cmd.verbose)
        logger = Logging.getLogger()

        logger.info(cmd)


        # ------------------------------------------------------------------------------------------------------------
        # run...

        if cmd.set:
            auth = MonitorAuth.from_user()

            if not auth.has_valid_email_address():
                logger.error("invalid email address: %s." % auth.email_address)
                exit(1)

            auth.save(Host, encryption_key=auth.password)

        elif cmd.delete:
            MonitorAuth.delete(Host)

        else:
            password = MonitorAuth.password_from_user()

            try:
                auth = MonitorAuth.load(Host, encryption_key=password)
            except (KeyError, ValueError):
                logger.error("incorrect password.")
                exit(1)


        # ------------------------------------------------------------------------------------------------------------
        # end...

        if auth:
            print(JSONify.dumps(auth))

    except KeyboardInterrupt:
        print(file=sys.stderr)



#!/usr/bin/env python3

"""
Created on 10 Jan 2022

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis

DESCRIPTION
The organisations utility is used to find, create, update or delete organisation records.

The utility supports a parent / child relationship between organisations - a child organisation can be
thought of as a subdivision of its parent. Users belonging to an organisation have the same rights over
child organisation data and devices as then have

SYNOPSIS
organisations.py [-c CREDENTIALS] {
-F [{ -l LABEL | -d ID }] [-m] |
-C -l LABEL -n LONG_NAME -u URL -o OWNER_EMAIL [-p PARENT_LABEL] |
-U LABEL [-l LABEL] [-n LONG_NAME] [-u URL] [-o OWNER_EMAIL] [-p PARENT_LABEL] |
-D LABEL } [-i INDENT] [-v]

EXAMPLES
organisations.py -F

DOCUMENT EXAMPLE
{"OrgID": 1, "Label": "SCS", "LongName": "South Coast Science", "URL": "https://www.southcoastscience.com",
"Owner": "bruno.beloff@southcoastscience.com", "ParentID": null}

DOCUMENT EXAMPLE - MEMBERSHIPS
{"organisation": {"OrgID": 11, "Label": "CEPEMAR", "LongName": "CEPEMAR", "URL": "http://cepemar.com",
"Owner": "felipe.tatagiba@cepemar.com", "ParentID": null}, "children": [
{"OrgID": 82, "Label": "Vale", "LongName": "Vale S.A.", "URL": "http://www.vale.com/",
"Owner": "felipe.tatagiba@cepemar.com", "ParentID": 11}]}

SEE ALSO
scs_analysis/cognito_user_credentials
"""

import sys

from scs_analysis.cmd.cmd_organisations import CmdOrganisations

from scs_core.aws.security.cognito_client_credentials import CognitoClientCredentials
from scs_core.aws.security.cognito_login_manager import CognitoLoginManager

from scs_core.aws.security.organisation import Organisation
from scs_core.aws.security.organisation_manager import OrganisationManager
from scs_core.aws.security.organisation_membership import OrganisationMembership

from scs_core.client.http_exception import HTTPException, HTTPNotFoundException

from scs_core.data.json import JSONify

from scs_core.sys.logging import Logging

from scs_host.comms.stdio import StdIO
from scs_host.sys.host import Host


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    logger = None
    report = None

    try:
        # ------------------------------------------------------------------------------------------------------------
        # cmd...

        cmd = CmdOrganisations()

        if not cmd.is_valid():
            cmd.print_help(sys.stderr)
            exit(2)

        Logging.config('organisations', verbose=cmd.verbose)       # level=logging.DEBUG
        logger = Logging.getLogger()

        logger.info(cmd)

        if cmd.label is not None and not Organisation.is_valid_label(cmd.label):
            logger.error("the label '%s' is not valid." % cmd.label)
            exit(2)

        if cmd.long_name is not None and not Organisation.is_valid_long_name(cmd.long_name):
            logger.error("the long name '%s' is not valid." % cmd.long_name)
            exit(2)

        if cmd.url is not None and not Organisation.is_valid_url(cmd.url):
            logger.error("the URL '%s' is not valid." % cmd.url)
            exit(2)

        if cmd.owner is not None and not Organisation.is_valid_owner(cmd.owner):
            logger.error("the owner email address '%s' is not valid." % cmd.owner)
            exit(2)


        # ------------------------------------------------------------------------------------------------------------
        # authentication...

        credentials = CognitoClientCredentials.load_for_user(Host, name=cmd.credentials_name)

        if not credentials:
            exit(1)

        gatekeeper = CognitoLoginManager()
        auth = gatekeeper.user_login(credentials)

        if not auth.is_ok():
            logger.error("login: %s." % auth.authentication_status.description)
            exit(1)


        # ------------------------------------------------------------------------------------------------------------
        # resources...

        manager = OrganisationManager()

        if cmd.parent_label is None or cmd.parent_label == 'none':
            parent_id = None

        else:
            try:
                parent = manager.get_organisation_by_label(auth.id_token, cmd.parent_label)
                parent_id = parent.org_id

            except HTTPNotFoundException:
                logger.error("no organisation found for parent label: '%s'." % cmd.parent_label)
                exit(2)


        # ------------------------------------------------------------------------------------------------------------
        # run...

        if cmd.find:
            if cmd.label is not None:
                report = manager.get_organisation_by_label(auth.id_token, cmd.label)
            elif cmd.id is not None:
                report = manager.get_organisation(auth.id_token, cmd.id)
            else:
                report = sorted(manager.find_organisations(auth.id_token))

            if cmd.memberships:
                if cmd.label is not None or cmd.id is not None:
                    report = [report]
                    orgs = manager.find_organisations(auth.id_token)
                else:
                    orgs = report

                report = OrganisationMembership.merge(report, orgs)

        if cmd.create:
            org = Organisation(0, cmd.label, cmd.long_name, cmd.url, cmd.owner, parent_id)
            report = manager.insert_organisation(auth.id_token, org)

        if cmd.update:
            # find...
            org = manager.get_organisation_by_label(auth.id_token, cmd.update)

            if org is None:
                logger.error("no organisation found for label: '%s'." % cmd.update)
                exit(1)

            # update...
            label = org.label if cmd.label is None else cmd.label
            long_name = org.long_name if cmd.long_name is None else cmd.long_name
            url = org.url if cmd.url is None else cmd.url
            owner = org.owner if cmd.owner is None else cmd.owner
            parent_id = org.parent_id if cmd.parent_label is None else parent_id

            report = Organisation(org.org_id, label, long_name, url, owner, parent_id)
            manager.update_organisation(auth.id_token, report)

        if cmd.delete:
            # find...
            org = manager.get_organisation_by_label(auth.id_token, cmd.delete)

            if org is None:
                logger.error("no organisation found for label: '%s'" % cmd.delete)
                exit(1)

            # check...
            response = StdIO.prompt('Are you sure? (Y/n)')
            if response != 'Y':
                exit(0)

            # delete...
            manager.delete_organisation(auth.id_token, org.org_id)


    # ----------------------------------------------------------------------------------------------------------------
    # end...

        if report is not None:
            print(JSONify.dumps(report, indent=cmd.indent))

        if cmd.find and cmd.label is not None and cmd.id is not None:
            logger.info("found: %s" % len(report))

    except KeyboardInterrupt:
        print(file=sys.stderr)

    except HTTPException as ex:
        logger.error(ex.error_report)
        exit(1)

    except Exception as ex:
        logger.error(ex.__class__.__name__)
        exit(1)



"""
Created on 22 Nov 2021

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)
"""

import optparse

from scs_analysis import version


# --------------------------------------------------------------------------------------------------------------------

class CmdCognitoEmail(object):
    """unix command line handler"""

    def __init__(self):
        """
        Constructor
        """
        self.__parser = optparse.OptionParser(usage="%prog { -e | -c | -s | -r } [-v] EMAIL",
                                              version=version())

        # operations...
        self.__parser.add_option("--send-email", "-e", action="store_true", dest="send_email",
                                 help="send access email")

        self.__parser.add_option("--confirm", "-c", action="store_true", dest="confirm",
                                 help="confirm account (using confirmation code)")

        self.__parser.add_option("--set-password", "-s", action="store_true", dest="set_password",
                                 help="set password (using temporary password)")

        self.__parser.add_option("--reset-password", "-r", action="store_true", dest="reset_password",
                                 help="reset password (using reset code)")

        # output...
        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    def is_valid(self):
        count = 0

        if self.send_email:
            count += 1

        if self.confirm:
            count += 1

        if self.set_password:
            count += 1

        if self.reset_password:
            count += 1

        if count != 1:
            return False

        if not self.__args or len(self.__args) != 1:
            return False

        return True


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def send_email(self):
        return self.__opts.send_email


    @property
    def confirm(self):
        return self.__opts.confirm


    @property
    def set_password(self):
        return self.__opts.set_password


    @property
    def reset_password(self):
        return self.__opts.reset_password


    @property
    def verbose(self):
        return self.__opts.verbose


    @property
    def email(self):
        return self.__args[0] if self.__args else None


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdCognitoEmail:{send_email:%s, confirm:%s, set_password:%s, reset:%s, verbose:%s}" % \
               (self.send_email, self.confirm, self.set_password, self.reset_password, self.verbose)



#!/usr/bin/env python3

"""
Created on 28 Jul 2021

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

DESCRIPTION
The baseline utility is use to set the baseline of a device's electrochemical and CO2 sensors according to estimated
low environmental gas concentrations. It does this by downloading device data, finding minimums, then adjusting the
device's baseline offsets by the difference between the estimated and reported values. Any changes in NO2
baselines are incorporated into O3 ('Ox') baselining.

Baselining can be applied to either:

* Val fields - including val.GGG.cnc and val.GGG.vCal (the model inputs)
* Exg fields - of the form exg.val.GGG.cnc or exg.MMM.GGG.cnc (the model outputs)

The device must be online during the baselining operation - a number of MQTT messages are passes between the
utility and the device. If any changes are made to the device's baseline settings, then the gases_sampler process
is restarted. How the restart happens is specified by the --uptake flag.

If the --rehearse flag is used, the baseline utility shows what changes would be made, but does not make the changes.

Operating parameters are specified by the baseline_conf utility, and may be overridden by the baseline utility flags.
Any number of named baseline_conf files may be stored.

SYNOPSIS
Usage: baseline.py [-c CREDENTIALS] -n CONF_NAME -f { V | E } [{ -r | -u COMMAND }] [-s START] [-e END]
[-p AGGREGATION] [-m GAS MINIMUM] [{ -o GAS | -x GAS }] [-v] DEVICE_TAG_1 .. DEVICE_TAG_N

EXAMPLES
baseline.py -c super -n freshfield -e 07:00 -fV scs-bgb-410

FILES
~/SCS/conf/baseline_conf/NAME_baseline_conf.json

SEE ALSO
scs_analysis/baseline_conf
scs_analysis/cognito_user_credentials

scs_dev/gases_sampler

scs_mfr/afe_baseline
scs_mfr/gas_baseline
scs_mfr/scd30_baseline
"""

import json
import sys

from scs_analysis.cmd.cmd_baseline import CmdBaseline

from scs_analysis.handler.batch_download_reporter import BatchDownloadReporter

from scs_core.aws.client.device_control_client import DeviceControlClient

from scs_core.aws.manager.byline.byline_finder import BylineFinder
from scs_core.aws.manager.topic_history.topic_history_manager import TopicHistoryManager

from scs_core.aws.security.cognito_client_credentials import CognitoClientCredentials
from scs_core.aws.security.cognito_login_manager import CognitoLoginManager

from scs_core.client.http_exception import HTTPGatewayTimeoutException

from scs_core.data.datetime import LocalizedDatetime
from scs_core.data.diurnal_period import DiurnalPeriod
from scs_core.data.json import JSONify
from scs_core.data.timedelta import Timedelta

from scs_core.estate.baseline_conf import BaselineConf
from scs_core.estate.configuration import Configuration

from scs_core.gas.afe_baseline import AFEBaseline
from scs_core.gas.afe_calib import AFECalib
from scs_core.gas.minimum import Minimum

from scs_core.sample.gases_sample import GasesSample

from scs_core.sys.logging import Logging

from scs_host.sys.host import Host


# TODO: review Ox handling
# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    MQTT_TIMEOUT = 60           # seconds

    key = None
    monitor_auth = None
    mqtt_client = None

    processed_count = 0

    # ----------------------------------------------------------------------------------------------------------------
    # cmd...

    cmd = CmdBaseline()

    if not cmd.is_valid():
        cmd.print_help(sys.stderr)
        exit(2)

    Logging.config('baseline', verbose=cmd.verbose)
    logger = Logging.getLogger()

    logger.info(cmd)


    try:
        # ------------------------------------------------------------------------------------------------------------
        # authentication...

        credentials = CognitoClientCredentials.load_for_user(Host, name=cmd.credentials_name)

        if not credentials:
            exit(1)

        gatekeeper = CognitoLoginManager()
        auth = gatekeeper.user_login(credentials)

        if not auth.is_ok():
            logger.error("login: %s." % auth.authentication_status.description)
            exit(1)


        # ------------------------------------------------------------------------------------------------------------
        # validation...

        if cmd.start is not None and not DiurnalPeriod.is_valid_time(cmd.start):
            logger.error("the start time is invalid.")
            exit(2)

        if cmd.end is not None and not DiurnalPeriod.is_valid_time(cmd.end):
            logger.error("the end time is invalid.")
            exit(2)


        # ------------------------------------------------------------------------------------------------------------
        # resources...

        # BylineFinder...
        reporter = BatchDownloadReporter()
        byline_finder = BylineFinder(reporter=reporter)

        # MessageManager...
        message_manager = TopicHistoryManager(reporter=reporter)

        # DeviceControlClient...
        client = DeviceControlClient()


        # ------------------------------------------------------------------------------------------------------------
        # run...

        for device_tag in cmd.device_tags:
            try:
                device_updates = 0

                # ----------------------------------------------------------------------------------------------------
                # configuration...

                logger.info("configuration...")

                # BaselineConf...
                baseline_conf = BaselineConf.load(Host, name=cmd.conf_name)

                if baseline_conf is None:
                    logger.error("the baseline configuration '%s' is not available." % cmd.conf_name)
                    continue

                try:
                    baseline_conf = cmd.override(baseline_conf)
                except KeyError as ex:
                    logger.error("the gas %s is not in the baseline configuration." % ex)
                    continue

                # host name...
                host_tag = Host.name()

                # topics...
                group = byline_finder.find_bylines_for_device(auth.id_token, device_tag)

                if not group:
                    logger.error("no bylines found for %s." % device_tag)
                    continue

                gases_topic = group.latest_topic(suffix='/gases')
                control_topic = group.latest_topic(suffix='/control')

                if gases_topic is None:
                    logger.error("no gases topic found for %s." % device_tag)
                    continue

                if control_topic is None:
                    logger.error("no control topic found for %s." % device_tag)
                    continue

                logger.info("gases_topic: %s" % gases_topic)
                logger.info("control_topic: %s" % control_topic)

                # configuration...
                response = client.interact(auth.id_token, device_tag, ['configuration'])
                command = response.command

                if command.return_code != 0:
                    logger.error("configuration cannot be retrieved: %s" % command.stderr[0])
                    continue

                jdict = json.loads(command.stdout[0])
                device_conf = Configuration.construct_from_jdict(jdict.get('val'))

                # noinspection PyUnresolvedReferences
                ox_index = None if device_conf.afe_id is None else device_conf.afe_id.sensor_index('Ox')

                # AFECalib...
                if ox_index is None:
                    ox_sensor = None

                else:
                    response = client.interact(auth.id_token, device_tag, ['afe_calib'])
                    command = response.command

                    if command.return_code != 0:
                        logger.error("AFECAlib cannot be retrieved: %s" % command.stderr[0])
                        continue

                    jdict = json.loads(command.stdout[0])
                    afe_calib = AFECalib.construct_from_jdict(jdict)

                    afe_baseline = AFEBaseline.null_datum() if device_conf.afe_baseline is None else \
                        device_conf.afe_baseline

                    ox_baseline = afe_baseline.sensor_baseline(ox_index)
                    ox_calib = afe_calib.sensor_calib(ox_index)

                    ox_sensor = ox_calib.sensor(ox_baseline)
                    # logger.error(ox_sensor)

                logger.error("-")


                # ----------------------------------------------------------------------------------------------------
                # analysis...

                # data...
                logger.info("data...")

                now = LocalizedDatetime.now()

                start = baseline_conf.start_datetime(now)
                end = baseline_conf.end_datetime(now)

                if start == end:
                    logger.error("the start and end hours may not be the same.")
                    exit(1)

                if end > now:
                    start -= Timedelta(days=1)
                    end -= Timedelta(days=1)
                    logger.error("WARNING: testing previous day...")

                logger.info("start: %s end: %s" % (start.as_iso8601(), end.as_iso8601()))

                data = list(message_manager.find_for_topic(auth.id_token, gases_topic, start, end,
                                                           None, False, baseline_conf.checkpoint(),
                                                           False, False, False, False, False, None))

                if not data:
                    logger.error("no data found for %s." % gases_topic)
                    continue

                logger.info("expected: %s retrieved: %s" % (baseline_conf.expected_data_points(start, end), len(data)))
                logger.info("-")

                # corrections...
                logger.info("correction...")

                conf_minimums = baseline_conf.minimums
                correction_applied = False
                no2_correction = None

                for minimum in Minimum.find_minimums(data, cmd.fields):
                    gas = minimum.gas

                    if cmd.excludes_gas(gas):       # not "only this gas"
                        continue

                    logger.info("-")
                    logger.info("%s..." % minimum.path)

                    if gas == cmd.exclude_gas:      # is excluded gas
                        logger.error("%s is excluded - skipping" % gas)
                        continue

                    if gas not in conf_minimums:
                        logger.error("%s has no specified minimum - skipping" % gas)
                        continue

                    # NO2...
                    if minimum.path == 'val.NO2.cnc':
                        no2_correction = conf_minimums['NO2'] - minimum.value

                    try:
                        if 'vCal' not in minimum.path and minimum.update_already_done(device_conf, end):
                            logger.error("%s has been updated since the latest test period - skipping" % minimum.path)
                            continue

                    except ValueError as ex:
                        logger.error("sensor with serial number %s is not supported - skipping" % ex)
                        continue

                    # Ox...
                    # TODO: correction needs to happen on the big data set - before minimums are found?
                    if minimum.path == 'val.Ox.cnc':
                        if no2_correction is None:
                            logger.error('NO2 minimum required for Ox, but none available - skipping')
                            continue

                        sample = GasesSample.construct_from_jdict(minimum.sample)

                        temp = sample.sht_datum.temp
                        no2_sample = sample.electrochem_datum.sns['NO2']
                        ox_sample = sample.electrochem_datum.sns['Ox']

                        no2_cnc = no2_sample.cnc + no2_correction
                        corrected = ox_sensor.datum(temp, ox_sample.we_v, ox_sample.ae_v, no2_cnc=no2_cnc)
                        minimum.value = corrected.cnc

                    # report...
                    print(JSONify.dumps(minimum.summary(gas)))
                    sys.stdout.flush()

                    # minimums...
                    if minimum.value == conf_minimums[gas]:
                        logger.error("%s matches the specified minimum - skipping" % minimum.path)
                        continue

                    if minimum.index == 0:
                        logger.error("WARNING: the first datum for %s is the minimum value" % minimum.path)

                    elif minimum.index == len(data) - 1:
                        logger.error("WARNING: the last datum for %s is the minimum value" % minimum.path)


                    # ------------------------------------------------------------------------------------------------
                    # update...

                    cmd_tokens = minimum.cmd_tokens(conf_minimums)
                    logger.info(' '.join([str(token) for token in cmd_tokens]))

                    if cmd.rehearse:
                        continue

                    response = client.interact(auth.id_token, device_tag, cmd_tokens)
                    command = response.command

                    if command.stderr:
                        print(*command.stderr, sep='\n', file=sys.stderr)
                    if command.stdout:
                        print(*command.stdout, sep='\n', file=sys.stdout)

                    if command.return_code != 0:
                        logger.error("update could not be performed: %s" % command.stderr[0])
                        continue

                    device_updates += 1

                # if not device_updates:
                #     continue

                # reboot...
                logger.info("-")

                logger.info(cmd.uptake)

                client.interact(auth.id_token, device_tag, [cmd.uptake])

            except HTTPGatewayTimeoutException:
                logger.error("device '%s' is not available." % device_tag)
                continue

            processed_count += 1


    # ----------------------------------------------------------------------------------------------------------------
    # end...

    except KeyboardInterrupt:
        print(file=sys.stderr)

    finally:
        logger.info("devices: %d processed: %d" % (len(cmd.device_tags), processed_count))



"""
Created on 29 Jun 2021

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)
"""

import optparse

from scs_analysis import version


# --------------------------------------------------------------------------------------------------------------------

class CmdClientTraffic(object):
    """unix command line handler"""

    # --------------------------------------------------------------------------------------------------------------------

    def __init__(self):
        """
        Constructor
        """
        self.__parser = optparse.OptionParser(usage="%prog [-c CREDENTIALS] [-e ENDPOINT] "
                                                    "{ -u | -o [-s] } -p PERIOD [-a] "
                                                    "[-i INDENT] [-v] [CLIENT_1..CLIENT_N]", version=version())

        # identity...
        self.__parser.add_option("--credentials", "-c", type="string", action="store", dest="credentials_name",
                                 help="the stored credentials to be presented")

        # filters...
        self.__parser.add_option("--endpoint", "-e", type="string", action="store", dest="endpoint",
                                 help="a specific endpoint (required if no clients)")

        self.__parser.add_option("--users", "-u", action="store_true", dest="user", default=False,
                                 help="a specific user")

        self.__parser.add_option("--organisations", "-o", action="store_true", dest="organisation", default=False,
                                 help="a specific organisation")

        self.__parser.add_option("--separate", "-s", action="store_true", dest="separate", default=False,
                                 help="report on each member individually")

        self.__parser.add_option("--period", "-p", type="string", action="store", dest="period",
                                 help="reporting period")

        self.__parser.add_option("--aggregate", "-a", action="store_true", dest="aggregate", default=False,
                                 help="aggregate data for the period")

        # output...
        self.__parser.add_option("--indent", "-i", type="int", action="store", dest="indent",
                                 help="pretty-print the output with INDENT")

        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    def is_valid(self):
        if self.user and self.organisation:
            return False

        if self.period is None:
            return False

        if self.separate and not self.organisation:
            return False

        if self.separate and len(self.clients) > 1:
            return False

        if not self.endpoint and not self.__args:
            return False

        return True


    # ----------------------------------------------------------------------------------------------------------------
    # properties: identity...

    @property
    def credentials_name(self):
        return self.__opts.credentials_name


    # ----------------------------------------------------------------------------------------------------------------
    # properties: filters...

    @property
    def endpoint(self):
        return self.__opts.endpoint


    @property
    def user(self):
        return self.__opts.user


    @property
    def organisation(self):
        return self.__opts.organisation


    @property
    def separate(self):
        return self.__opts.separate


    @property
    def period(self):
        return self.__opts.period


    @property
    def aggregate(self):
        return self.__opts.aggregate


    @property
    def clients(self):
        return self.__args


    # ----------------------------------------------------------------------------------------------------------------
    # properties: output...

    @property
    def indent(self):
        return self.__opts.indent


    @property
    def verbose(self):
        return self.__opts.verbose


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdClientTraffic:{credentials_name:%s, endpoint:%s, user:%s, organisation:%s, separate:%s, " \
               "period:%s, aggregate:%s, indent:%s, verbose:%s, clients:%s}" % \
               (self.credentials_name, self.endpoint, self.user, self.organisation, self.separate,
                self.period, self.aggregate, self.indent, self.verbose, self.clients)



#!/usr/bin/env python3

"""
Created on 28 Jun 2023

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

DESCRIPTION
The device monitor periodically checks on the availability and health of every air quality monitoring device. The
device_monitor_status utility is used to deliver the most recent report for the specified device(s).

SYNOPSIS
device_monitor_status.py [-c CREDENTIALS] [-t DEVICE_TAG [-x]] [-i INDENT] [-v]

EXAMPLES
device_monitor_status.py -vi4 -c super -t scs-bgx-570 -x

DOCUMENT EXAMPLE
{
    "scs-bgx-570": {
        "device-tag": "scs-bgx-570",
        "availability": {
            "is-ok": true,
            "since": "2023-01-16T13:45:42Z"
        },
        "data": {
            "south-coast-science-test/pmx/device/praxis-000570/status": {
                "is-ok": false,
                "since": "2021-07-30T11:30:43Z"
            },
            "south-coast-science-test/pmx/loc/570/climate": {
                "is-ok": false,
                "since": "2021-07-30T11:45:42Z"
            },
            "south-coast-science-test/pmx/loc/570/particulates": {
                "is-ok": false,
                "since": "2021-07-30T12:00:43Z"
            }
        },
        "power": {
            "is-ok": true,
            "since": "2023-01-16T14:15:42Z"
        },
        "uptime": {
            "period": null
        }
    }
}

SEE ALSO
scs_analysis/cognito_user_credentials
scs_analysis/device_monitor
"""

import sys

from scs_analysis.cmd.cmd_device_monitor_status import CmdDeviceMonitorStatus

from scs_core.aws.monitor.device.device_monitor_status_manager import DeviceMonitorStatusManager

from scs_core.aws.security.cognito_client_credentials import CognitoClientCredentials
from scs_core.aws.security.cognito_device import CognitoDeviceCredentials
from scs_core.aws.security.cognito_login_manager import CognitoLoginManager

from scs_core.client.http_exception import HTTPException

from scs_core.data.json import JSONify

from scs_core.sys.logging import Logging

from scs_host.sys.host import Host


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    logger = None
    response = None
    report = None

    try:
        # ------------------------------------------------------------------------------------------------------------
        # cmd...

        cmd = CmdDeviceMonitorStatus()

        if not cmd.is_valid():
            cmd.print_help(sys.stderr)
            exit(2)

        Logging.config('device_monitor_status', verbose=cmd.verbose)
        logger = Logging.getLogger()

        logger.info(cmd)


        # ------------------------------------------------------------------------------------------------------------
        # authentication...

        credentials = CognitoClientCredentials.load_for_user(Host, name=cmd.credentials_name)

        if not credentials:
            exit(1)

        gatekeeper = CognitoLoginManager()
        auth = gatekeeper.user_login(credentials)

        if not auth.is_ok():
            logger.error("login: %s." % auth.authentication_status.description)
            exit(1)


        # ------------------------------------------------------------------------------------------------------------
        # validation...

        if cmd.tag_filter is not None and cmd.exact_match and not CognitoDeviceCredentials.is_valid_tag(cmd.tag_filter):
            logger.error("The device tag '%s' is not valid." % cmd.tag_filter)
            exit(2)


        # ------------------------------------------------------------------------------------------------------------
        # resources...

        manager = DeviceMonitorStatusManager()


        # ------------------------------------------------------------------------------------------------------------
        # run...

        report = manager.find(auth.id_token, device_tag_filter=cmd.tag_filter, exact=cmd.exact_match)


        # ------------------------------------------------------------------------------------------------------------
        # end...

        # report...
        if report is not None:
            print(JSONify.dumps(report, indent=cmd.indent))
            logger.info('retrieved: %s' % len(report))

    except KeyboardInterrupt:
        print(file=sys.stderr)

    except HTTPException as ex:
        logger.error(ex.error_report)
        exit(1)

    except Exception as ex:
        logger.error(ex.__class__.__name__)
        exit(1)



"""
Created on 29 Jun 2021

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)
"""

import optparse

from scs_analysis import version

from scs_core.data.diurnal_period import DiurnalPeriod
from scs_core.data.recurring_period import RecurringPeriod

from scs_core.email.email import EmailRecipient

from scs_core.location.timezone import Timezone


# --------------------------------------------------------------------------------------------------------------------

class CmdAlert(object):
    """unix command line handler"""

    def __init__(self):
        """
        Constructor
        """
        self.__parser = optparse.OptionParser(usage="%prog { -z | [-c CREDENTIALS]  "
                                                    "{ -F | -R ID | -C | -U ID | -D ID } "
                                                    "[-d DESCRIPTION] [-p TOPIC] [-f FIELD] [-l LOWER] [-u UPPER] "
                                                    "[-n { 0 | 1 }] "
                                                    "[{ -r INTERVAL UNITS TIMEZONE | -t START END TIMEZONE }] "
                                                    "[-a { 0 | 1 }] [-s { 0 | 1 }] [-i INDENT] [-v] "
                                                    "[-e EMAIL_ADDR] [-b { A | R } EMAIL_ADDR [-j]] }",
                                              version=version())

        # identity...
        self.__parser.add_option("--credentials", "-c", type="string", action="store", dest="credentials_name",
                                 help="the stored credentials to be presented")

        # operations...
        self.__parser.add_option("--zones", "-z", action="store_true", dest="list", default=False,
                                 help="list the available timezone names to stderr")

        self.__parser.add_option("--find", "-F", action="store_true", dest="find", default=False,
                                 help="find alerts for given description, topic, field or email")

        self.__parser.add_option("--retrieve", "-R", type="int", action="store", dest="retrieve_id",
                                 help="retrieve alert with given ID")

        self.__parser.add_option("--create", "-C", action="store_true", dest="create", default=False,
                                 help="create alert")

        self.__parser.add_option("--update", "-U", type="int", action="store", dest="update_id",
                                 help="update alert with given ID")

        self.__parser.add_option("--delete", "-D", type="int", action="store", dest="delete_id",
                                 help="delete alert with given ID")

        # fields...
        self.__parser.add_option("--description", "-d", type="string", action="store", dest="description",
                                 default="", help="description")

        self.__parser.add_option("--topic-path", "-p", type="string", action="store", dest="topic",
                                 help="topic path")

        self.__parser.add_option("--field", "-f", type="string", action="store", dest="field",
                                 help="field")

        self.__parser.add_option("--lower-threshold", "-l", type="float", action="store", dest="lower_threshold",
                                 help="lower threshold")

        self.__parser.add_option("--upper-threshold", "-u", type="float", action="store", dest="upper_threshold",
                                 help="upper threshold")

        self.__parser.add_option("--alert-on-none", "-n", type="int", action="store", dest="alert_on_none",
                                 default=False, help="alert on none (default false)")

        self.__parser.add_option("--recurring-period", "-r", type="string", nargs=3, action="store",
                                 dest="recurring_period",
                                 help="aggregation interval, units { D | H | M } and timezone")

        self.__parser.add_option("--diurnal-period", "-t", type="string", nargs=3, action="store",
                                 dest="diurnal_period", help="aggregation start, end and timezone")

        self.__parser.add_option("--contiguous-alerts", "-a", type="int", action="store",
                                 dest="contiguous_alerts", default=None,
                                 help="raise alert on contiguous exceedence (default true)")

        self.__parser.add_option("--suspended", "-s", type="int", action="store", dest="suspended",
                                 default=None, help="suspended (default false)")

        # email...
        self.__parser.add_option("--email", "-e", type="string", action="store", dest="email",
                                 help="email To address (any on find)")

        self.__parser.add_option("--bcc-list", "-b", type="string", nargs=2, action="store", dest="bcc", default=False,
                                 help="Add or Remove from BCC list")

        self.__parser.add_option("--json-message", "-j", action="store_true", dest="json_message", default=False,
                                 help="message body is JSON")

        # output...
        self.__parser.add_option("--indent", "-i", type="int", action="store", dest="indent",
                                 help="pretty-print the output with INDENT")

        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    def is_valid(self):
        count = 0

        if self.list:
            count += 1

        if self.find:
            count += 1

        if self.retrieve:
            count += 1

        if self.create:
            count += 1

        if self.update:
            count += 1

        if self.delete:
            count += 1

        if count != 1:
            return False

        if self.alert_on_none is not None and self.alert_on_none != 0 and self.alert_on_none != 1:
            return False

        if self.json_message is not None and self.json_message != 0 and self.json_message != 1:
            return False

        if self.contiguous_alerts is not None and \
                self.contiguous_alerts != 0 and self.contiguous_alerts != 1:
            return False

        if self.suspended is not None and self.suspended != 0 and self.suspended != 1:
            return False

        if self.bcc and self.bcc_action not in ['A', 'R']:
            return False

        if self.__args:
            return False

        return True


    def is_complete(self):
        if not self.create:
            return True

        if self.topic is None or self.field is None or self.aggregation_period is None:
            return False

        if self.lower_threshold is None and self.upper_threshold is None and not self.alert_on_none:
            return False

        return True


    def is_valid_start_time(self):
        if self.__opts.diurnal_period is None:
            return True

        return DiurnalPeriod.is_valid_time(self.__opts.diurnal_period[0])


    def is_valid_end_time(self):
        if self.__opts.diurnal_period is None:
            return True

        return DiurnalPeriod.is_valid_time(self.__opts.diurnal_period[1])


    def is_valid_timezone(self):
        if self.__opts.recurring_period is not None:
            return Timezone.is_valid(self.__opts.recurring_period[2])

        if self.__opts.diurnal_period is not None:
            return Timezone.is_valid(self.__opts.diurnal_period[2])

        return True


    def updated_bcc_dict(self, existing_bcc_dict):
        if not self.bcc:
            return existing_bcc_dict

        if self.bcc[0] == 'A':
            existing_bcc_dict[self.bcc[1]] = EmailRecipient(self.bcc[1], self.json_message)
            return existing_bcc_dict

        if self.bcc[0] == 'R':
            del existing_bcc_dict[self.bcc[1]]
            return existing_bcc_dict


    # ----------------------------------------------------------------------------------------------------------------
    # properties: identity...

    @property
    def credentials_name(self):
        return self.__opts.credentials_name


    @property
    def id(self):
        if self.__opts.retrieve_id is not None:
            return self.__opts.retrieve_id

        if self.__opts.update_id is not None:
            return self.__opts.update_id

        if self.__opts.delete_id is not None:
            return self.__opts.delete_id

        return None


    # ----------------------------------------------------------------------------------------------------------------
    # properties: operations...

    @property
    def list(self):
        return self.__opts.list


    @property
    def find(self):
        return self.__opts.find


    @property
    def retrieve(self):
        return self.__opts.retrieve_id is not None


    @property
    def create(self):
        return self.__opts.create


    @property
    def update(self):
        return self.__opts.update_id is not None


    @property
    def delete(self):
        return self.__opts.delete_id is not None


    # ----------------------------------------------------------------------------------------------------------------
    # properties: fields...

    @property
    def description(self):
        return self.__opts.description


    @property
    def topic(self):
        return self.__opts.topic


    @property
    def field(self):
        return self.__opts.field


    @property
    def lower_threshold(self):
        return self.__opts.lower_threshold


    @property
    def upper_threshold(self):
        return self.__opts.upper_threshold


    @property
    def alert_on_none(self):
        return self.__opts.alert_on_none


    @property
    def aggregation_period(self):
        return self.diurnal_period if self.recurring_period is None else self.recurring_period


    @property
    def recurring_period(self):
        period = self.__opts.recurring_period
        return None if period is None else RecurringPeriod.construct(period[0], period[1], period[2])


    @property
    def diurnal_period(self):
        period = self.__opts.diurnal_period
        return None if period is None else DiurnalPeriod.construct(period[0], period[1], period[2])


    @property
    def contiguous_alerts(self):
        return self.__opts.contiguous_alerts


    @property
    def json_message(self):
        return self.__opts.json_message


    @property
    def suspended(self):
        return self.__opts.suspended


    # ----------------------------------------------------------------------------------------------------------------
    # properties: email...

    @property
    def email(self):
        return self.__opts.email


    @property
    def bcc(self):
        return self.__opts.bcc


    @property
    def bcc_action(self):
        return None if not self.__opts.bcc else self.__opts.bcc[0]


    @property
    def bcc_email(self):
        return None if not self.__opts.bcc else self.__opts.bcc[1]


    # ----------------------------------------------------------------------------------------------------------------
    # fields: output...

    @property
    def indent(self):
        return self.__opts.indent


    @property
    def verbose(self):
        return self.__opts.verbose


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdAlert:{list:%s, credentials_name:%s, find:%s, retrieve:%s, create:%s, " \
               "update:%s, delete:%s, topic:%s, field:%s, lower_threshold:%s, " \
               "upper_threshold:%s, alert_on_none:%s, recurring_period:%s, diurnal_period:%s, " \
               "json_message:%s, json_message:%s, suspended:%s, email:%s, bcc:%s, " \
               "indent:%s, verbose:%s}" % \
               (self.list, self.credentials_name, self.find, self.__opts.retrieve_id, self.create,
                self.__opts.update_id, self.__opts.delete_id, self.topic, self.field, self.lower_threshold,
                self.upper_threshold, self.alert_on_none, self.__opts.recurring_period, self.__opts.diurnal_period,
                self.contiguous_alerts, self.json_message, self.suspended, self.email, self.bcc,
                self.indent, self.verbose)



#!/usr/bin/env python3

"""
Created on 3 Jan 2022

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis

DESCRIPTION
The sample_time_shift utility is used to add a unit of time to the reported datetime of each record in the input
data set. This is useful when reference data is labelled for the beginning of the time period instead of the end
(the latter is the form used by South Coast Science).

The path to the ISO 8601 recording datetime may be given, otherwise it defaults to 'rec'.

SYNOPSIS
sample_time_shift.py -t { + | - } [[DD-]HH:]MM[:SS] [-v] [PATH]

EXAMPLES
csv_reader.py -v mfidikwe.csv | sample_time_shift.py -v -t + 00:01:00 | csv_writer.py -v mfidikwe_shifted.csv

DOCUMENT EXAMPLE - INPUT
{"rec": "2021-10-31T22:00:00Z", "meteo": {"T": 15.2, "rH": 62}, "gas": {"SO2": 11}}
...

DOCUMENT EXAMPLE - OUTPUT
{"rec": "2021-10-31T22:01:00Z", "meteo": {"T": 15.2, "rH": 62}, "gas": {"SO2": 11}}
...

SEE ALSO
scs_analysis/sample_iso_8601
scs_analysis/sample_localize
"""

import sys

from scs_analysis.cmd.cmd_sample_time_shift import CmdSampleTimeShift

from scs_core.data.datetime import LocalizedDatetime
from scs_core.data.json import JSONify
from scs_core.data.path_dict import PathDict

from scs_core.sys.logging import Logging


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    document_count = 0
    processed_count = 0

    # ----------------------------------------------------------------------------------------------------------------
    # cmd...

    cmd = CmdSampleTimeShift()

    Logging.config('sample_time_shift', verbose=cmd.verbose)
    logger = Logging.getLogger()

    if not cmd.is_valid_timedelta():
        logger.error("invalid format for timedelta.")
        exit(2)

    if not cmd.is_valid():
        cmd.print_help(sys.stderr)
        exit(2)

    logger.info(cmd)

    try:
        # ------------------------------------------------------------------------------------------------------------
        # run...

        for line in sys.stdin:
            datum = PathDict.construct_from_jstr(line)

            if datum is None:
                break

            document_count += 1

            rec = LocalizedDatetime.construct_from_iso8601(datum.node(cmd.path))
            shifted = rec + cmd.timedelta if cmd.positive else rec - cmd.timedelta

            datum.append(cmd.path, shifted.as_iso8601())

            print(JSONify.dumps(datum))
            sys.stdout.flush()

            processed_count += 1


    # ----------------------------------------------------------------------------------------------------------------
    # end...

    except KeyboardInterrupt:
        print(file=sys.stderr)

    except KeyError as ex:
        logger.error(repr(ex))
        exit(1)

    finally:
        logger.info("documents: %d processed: %d" % (document_count, processed_count))



"""
Created on 16 Mar 2017

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis
"""

import optparse

from scs_analysis import version


# --------------------------------------------------------------------------------------------------------------------

class CmdLowPass(object):
    """
    unix command line handler
    """

    def __init__(self):
        """
        Constructor
        """
        self.__parser = optparse.OptionParser(usage="%prog -d DELTA_T -c CUT_OFF [-p PRECISION] [-v] [PATH]",
                                              version=version())

        # compulsory...
        self.__parser.add_option("--delta", "-d", type="float", action="store", dest="delta",
                                 help="sampling time interval")

        self.__parser.add_option("--cut-off", "-c", type="float", action="store", dest="cut_off",
                                 help="cut-off frequency")

        # output...
        self.__parser.add_option("--prec", "-p", type="int", action="store", default=None, dest="precision",
                                 help="precision (default 0 decimal places)")

        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    def is_valid(self):
        if self.delta is None or self.cut_off is None:
            return False

        if self.__args and len(self.__args) != 1:
            return False

        return True


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def delta(self):
        return self.__opts.delta


    @property
    def cut_off(self):
        return self.__opts.cut_off


    @property
    def precision(self):
        return self.__opts.precision


    @property
    def verbose(self):
        return self.__opts.verbose


    @property
    def path(self):
        return self.__args[0] if len(self.__args) > 0 else None


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdLowPassFilter:{delta:%s, cut_off:%s, precision:%s, verbose:%s, path:%s}" % \
               (self.delta, self.cut_off, self.precision, self.verbose, self.path)



"""
Created on 19 Jan 2022

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)
"""

import optparse

from scs_analysis import version


# --------------------------------------------------------------------------------------------------------------------

class CmdOrganisationDevices(object):
    """unix command line handler"""

    def __init__(self):
        """
        Constructor
        """
        self.__parser = optparse.OptionParser(usage="%prog [-c CREDENTIALS] { -F { -l ORG_LABEL | -t DEVICE_TAG } | "
                                                    "-C -l ORG_LABEL -t DEVICE_TAG -p PATH_ROOT GROUP LOCATION"
                                                    " -d DEPLOYMENT_LABEL | -D -t DEVICE_TAG } "
                                                    "[-i INDENT] [-v]", version=version())

        # identity...
        self.__parser.add_option("--credentials", "-c", type="string", action="store", dest="credentials_name",
                                 help="the stored credentials to be presented")

        # operations...
        self.__parser.add_option("--Find", "-F", action="store_true", dest="find", default=False,
                                 help="find devices for the given organisation or device tag")

        self.__parser.add_option("--Create", "-C", action="store_true", dest="create", default=False,
                                 help="create a device")

        self.__parser.add_option("--Delete", "-D", action="store_true", dest="delete", default=False,
                                 help="delete the device")

        # fields...
        self.__parser.add_option("--org-label", "-l", type="string", action="store", dest="org_label",
                                 help="the organisation label")

        self.__parser.add_option("--device-tag", "-t", type="string", action="store", dest="device_tag",
                                 help="the device tag")

        self.__parser.add_option("--project", "-p", type="string", nargs=3, action="store", dest="project",
                                 help="path root, group and location number")

        self.__parser.add_option("--deployment-label", "-d", type="string", action="store", dest="deployment_label",
                                 help="the device's deployment label")

        # output...
        self.__parser.add_option("--indent", "-i", type="int", action="store", dest="indent",
                                 help="pretty-print the output with INDENT")

        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    def is_valid(self):
        count = 0

        if self.find:
            count += 1

        if self.create:
            count += 1

        if self.delete:
            count += 1

        if count != 1:
            return False

        if self.find and self.org_label is None and self.device_tag is None:
            return False

        if (self.create or self.delete) and (self.org_label is None or self.device_tag is None):
            return False

        if self.delete and (self.__opts.project is None):
            return False

        if self.create and (self.__opts.project is None or self.deployment_label is None):
            return False

        if self.__args:
            return False

        return True


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def credentials_name(self):
        return self.__opts.credentials_name


    @property
    def find(self):
        return self.__opts.find


    @property
    def create(self):
        return self.__opts.create


    @property
    def delete(self):
        return self.__opts.delete


    @property
    def org_label(self):
        return self.__opts.org_label


    @property
    def device_tag(self):
        return self.__opts.device_tag


    @property
    def project_organisation(self):
        return None if self.__opts.project is None else self.__opts.project[0]


    @property
    def project_group(self):
        return None if self.__opts.project is None else self.__opts.project[1]


    @property
    def project_location(self):
        return None if self.__opts.project is None else self.__opts.project[2]


    @property
    def deployment_label(self):
        return self.__opts.deployment_label


    @property
    def indent(self):
        return self.__opts.indent


    @property
    def verbose(self):
        return self.__opts.verbose


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdOrganisationDevices:{credentials_name:%s, find:%s, create:%s, delete:%s, " \
               "org_label:%s, device_tag:%s, project:%s, deployment_label:%s, indent:%s, verbose:%s}" % \
               (self.credentials_name, self.find, self.create, self.delete,
                self.org_label, self.device_tag, self.__opts.project, self.deployment_label, self.indent, self.verbose)



"""
Created on 18 Jan 2022

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)
"""

import optparse

from scs_analysis import version


# --------------------------------------------------------------------------------------------------------------------

class CmdOrganisationUserPaths(object):
    """unix command line handler"""

    def __init__(self):
        """
        Constructor
        """
        self.__parser = optparse.OptionParser(usage="%prog  [-c CREDENTIALS] { -F { -e EMAIL | -r PATH_ROOT } | "
                                                    "-C -e EMAIL -r PATH_ROOT -x PATH_EXTENSION | "
                                                    "-D -e EMAIL -r PATH_ROOT -x PATH_EXTENSION } "
                                                    "[-i INDENT] [-v]", version=version())

        # identity...
        self.__parser.add_option("--credentials", "-c", type="string", action="store", dest="credentials_name",
                                 help="the stored credentials to be presented")

        # operations...
        self.__parser.add_option("--Find", "-F", action="store_true", dest="find", default=False,
                                 help="find users for the given username or organisation")

        self.__parser.add_option("--Create", "-C", action="store_true", dest="create", default=False,
                                 help="create a user")

        self.__parser.add_option("--Delete", "-D", action="store_true", dest="delete", default=False,
                                 help="delete the user for the given username and organisation")

        # fields...
        self.__parser.add_option("--email", "-e", type="string", action="store", dest="email",
                                 help="the user's email address'")

        self.__parser.add_option("--path-root", "-r", type="string", action="store", dest="path_root",
                                 help="the organisation path root")

        self.__parser.add_option("--path-extension", "-x", type="string", action="store", dest="path_extension",
                                 help="the organisation path root")

        # output...
        self.__parser.add_option("--indent", "-i", type="int", action="store", dest="indent",
                                 help="pretty-print the output with INDENT")

        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    def is_valid(self):
        count = 0

        if self.find:
            count += 1

        if self.create:
            count += 1

        if self.delete:
            count += 1

        if count != 1:
            return False

        if self.find:
            if self.email is None and self.path_root is None:
                return False
        else:
            if self.email is None or self.path_root is None:
                return False

        if (self.create or self.delete) and self.path_extension is None:
            return False

        if self.__args:
            return False

        return True


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def credentials_name(self):
        return self.__opts.credentials_name


    @property
    def find(self):
        return self.__opts.find


    @property
    def create(self):
        return self.__opts.create


    @property
    def delete(self):
        return self.__opts.delete


    @property
    def email(self):
        return self.__opts.email


    @property
    def path_root(self):
        return self.__opts.path_root


    @property
    def path_extension(self):
        return self.__opts.path_extension


    @property
    def indent(self):
        return self.__opts.indent


    @property
    def verbose(self):
        return self.__opts.verbose


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdOrganisationUserPaths:{credentials_name:%s, find:%s, create:%s, delete:%s, " \
               "email:%s, path_root:%s, path_extension:%s, indent:%s, verbose:%s}" % \
               (self.credentials_name, self.find, self.create, self.delete,
                self.email, self.path_root, self.path_extension, self.indent, self.verbose)



"""
Created on 23 Oct 2020

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis
"""

import optparse

from scs_analysis import version
from scs_core.data.timedelta import Timedelta


# --------------------------------------------------------------------------------------------------------------------

class CmdSampleSlope(object):
    """unix command line handler"""

    def __init__(self):
        """
        Constructor
        """
        self.__parser = optparse.OptionParser(usage="%prog -n NAME [-i ISO] [-t TALLY] [-m [DD-]HH:MM[:SS]] [-x] "
                                                    "[-p PRECISION] [-v] PATH", version=version())

        # identity...
        self.__parser.add_option("--name", "-n", type="string", action="store", dest="name",
                                 help="slope field name (i.e. '1min')")

        # mode...
        self.__parser.add_option("--iso-path", "-i", type="string", action="store", default="rec", dest="iso",
                                 help="path for ISO 8601 datetime field (default 'rec')")

        self.__parser.add_option("--tally", "-t", type="int", action="store", default=2, dest="tally",
                                 help="compute for rolling TALLY number of data points (default 2)")

        self.__parser.add_option("--max-interval", "-m", type="string", action="store", dest="max_interval",
                                 help="restart regression on long intervals")

        self.__parser.add_option("--exclude-incomplete", "-x", action="store_true", dest="exclude_incomplete",
                                 default=False, help="exclude incomplete tallies")

        # output...
        self.__parser.add_option("--prec", "-p", type="int", action="store", default=6, dest="precision",
                                 help="precision (default 6 decimal places)")

        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    def is_valid(self):
        if self.name is None:
            return False

        if self.tally < 1:
            return False

        if self.path is None:
            return False

        return True


    def is_valid_interval(self):
        if self.__opts.max_interval is None:
            return True

        return self.max_interval is not None


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def name(self):
        return self.__opts.name


    @property
    def iso(self):
        return self.__opts.iso


    @property
    def tally(self):
        return self.__opts.tally


    @property
    def max_interval(self):
        return Timedelta.construct_from_flag(self.__opts.max_interval)


    @property
    def exclude_incomplete(self):
        return self.__opts.exclude_incomplete


    @property
    def precision(self):
        return self.__opts.precision


    @property
    def verbose(self):
        return self.__opts.verbose


    @property
    def path(self):
        return self.__args[0] if len(self.__args) > 0 else None


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdSampleSlope:{name:%s, iso:%s, tally:%s, exclude_incomplete:%s, precision:%s, verbose:%s, " \
               "path:%s}" % \
               (self.name, self.iso, self.tally, self.exclude_incomplete, self.precision, self.verbose,
                self.path)



"""
Created on 19 Sep 2022

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis
"""

import optparse

from scs_analysis import version


# --------------------------------------------------------------------------------------------------------------------

class CmdSampleSort(object):
    """unix command line handler"""

    def __init__(self):
        """
        Constructor
        """
        self.__parser = optparse.OptionParser(usage="%prog [-v] SORT_PATH_1 [..SORT_PATH_N]", version=version())

        # output...
        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    def is_valid(self):
        if len(self.__args) < 1:
            return False

        return True


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def verbose(self):
        return self.__opts.verbose


    @property
    def sort_paths(self):
        return self.__args


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdSampleSort:{verbose:%s, sort_paths:%s}" % (self.verbose, self.sort_paths)



#!/usr/bin/env python3

"""
Created on 18 Sep 2021

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_exegesis

DESCRIPTION
The sample_paths utility is used to

Typically use with csv_reader --limit=1 flag.

SYNOPSIS
sample_paths.py [-v]

EXAMPLES
csv_reader.py -v -l 1 ref-scs-opc-116-gases-2021H1-vcal-slp-err-vE-exg.csv | sample_paths.py
"""

import sys

from scs_analysis.cmd.cmd_sample_paths import CmdSamplePaths

from scs_core.data.json import JSONify
from scs_core.data.path_dict import PathDict

from scs_core.sys.logging import Logging


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    paths = None

    # ----------------------------------------------------------------------------------------------------------------
    # cmd...

    cmd = CmdSamplePaths()

    if not cmd.is_valid():
        cmd.print_help(sys.stderr)
        exit(2)

    Logging.config('sample_paths', verbose=cmd.verbose)
    logger = Logging.getLogger()

    logger.info(cmd)


    try:
        # ------------------------------------------------------------------------------------------------------------
        # run...

        for line in sys.stdin:
            if paths:
                continue

            datum = PathDict.construct_from_jstr(line)

            if datum is None:
                continue

            paths = datum.paths()


        # report...
        print(JSONify.dumps(paths))
        sys.stdout.flush()


    # ----------------------------------------------------------------------------------------------------------------
    # end...

    except KeyboardInterrupt:
        print(file=sys.stderr)



"""
Created on 21 Jul 2016

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis

uses matplotlibrc configuration file

https://matplotlib.org/faq/usage_faq.html
https://matplotlib.org/stable/users/interactive_guide.html
https://stackoverflow.com/questions/28269157/plotting-in-a-non-blocking-way-with-matplotlib?noredirect=1&lq=1
http://physicalmodelingwithpython.blogspot.com/2015/07/raising-figure-window-to-foreground.html

Deprecation warnings with Matplotlib 3.4 #383
https://github.com/raysect/source/issues/383
"""

from matplotlib import pyplot as plt

from scs_analysis.chart.chart import Chart


# TODO: move the Y baseline up if zero is not needed
# --------------------------------------------------------------------------------------------------------------------

class SingleChart(Chart):
    """
    classdocs
    """

    def __init__(self, title, batch_mode, x_count, y_min, y_max, is_relative, path):
        """
        Constructor
        """
        super().__init__(title, batch_mode)

        # fields...
        self.__y_min = y_min
        self.__y_max = y_max
        self.__is_relative = is_relative

        self.__path = path

        self.__index = 1
        self.__first_datum = None

        initial_datum = 0 if is_relative else y_min
        self.__y_data = [initial_datum] * x_count

        # plotter...
        fig = plt.figure()
        # fig.canvas.set_window_title(self.title(path))                     # deprecated
        fig.canvas.manager.set_window_title(self.title(path))
        fig.tight_layout()

        fig.canvas.mpl_connect('close_event', self.close)

        ax1 = plt.axes()
        ax1.xaxis.grid(True)

        plt.grid()

        plt.ylim([y_min, y_max])

        if not batch_mode:
            self.__line, = plt.plot(self.__y_data)


    # ----------------------------------------------------------------------------------------------------------------

    def plot(self, path_dict):
        if self.closed:
            return

        # datum...
        try:
            value = path_dict.node(self.__path)
        except KeyError:
            return

        if value is None:
            return

        datum = float(value)

        # adjust...
        if self.__is_relative:
            if self.__first_datum is None:
                self.__first_datum = datum
                display_datum = 0
            else:
                display_datum = datum - self.__first_datum
        else:
            display_datum = datum

        # update data...
        del self.__y_data[0]
        self.__y_data.append(display_datum)

        if not self.batch_mode:
            self.__line.set_ydata(self.__y_data)

        # min / max...
        min_data_y = float(min(self.__y_data))
        max_data_y = float(max(self.__y_data))

        min_axis_y = self.__y_min if min_data_y > self.__y_min else min_data_y - (abs(min_data_y) * 0.05)
        max_axis_y = self.__y_max if max_data_y < self.__y_max else max_data_y + (abs(max_data_y) * 0.05)

        plt.ylim([min_axis_y, max_axis_y])

        if not self.batch_mode:
            self.render(block=False)


    def render(self, block=True):
        if self.batch_mode:
            plt.plot(self.__y_data)

        plt.draw()
        plt.show(block=block)
        plt.pause(0.001)


    def close(self, _event):
        self._closed = True
        plt.close()


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def y_min(self):
        return self.__y_min


    @property
    def y_max(self):
        return self.__y_max


    @property
    def index(self):
        return self.__index


    @property
    def first_datum(self):
        return self.__first_datum


    @property
    def y_data(self):
        return [format(datum, '.6f') for datum in self.__y_data]


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "SingleChart:{batch_mode:%s, y_min:%s, y_max:%s, is_relative:%s, path:%s, index:%s, first_datum:%s}" % \
                (self.batch_mode, self.y_min, self.y_max, self.__is_relative, self.__path, self.index,
                 self.first_datum)



#!/usr/bin/env python3

"""
Created on 20 Mar 2018

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis

DESCRIPTION
The sample_median utility provides a median filter - the filter maintains an odd-numbered window on the input
data sequence, and outputs the middle item in the sorted list of items. The user can specify the
size of the window.

Input data is typically in the form of a JSON document. A command parameter specifies the path to the node within
the document that is to be filtered. The node is typically a leaf node integer or float. The output of the
sample_median utility includes the source value, and the smoothed value.

SYNOPSIS
sample_median.py [-w SIZE] [-p PRECISION] [-v] [PATH]

EXAMPLES
aws_topic_history.py -m60 /orgs/south-coast-science-demo/brighton/loc/1/gases | \
sample_median.py -w 3 -p 1 val.CO.cnc

DOCUMENT EXAMPLE - INPUT
{"tag": "scs-bgx-401", "rec": "2018-03-27T09:54:41.042+00:00", "val": {
"NO2": {"weV": 0.29563, "aeV": 0.280879, "weC": 0.009569, "cnc": 61.0},
"Ox": {"weV": 0.406819, "aeV": 0.387443, "weC": -0.010706, "cnc": 34.7},
"NO": {"weV": 0.319692, "aeV": 0.292129, "weC": 0.028952, "cnc": 165.5},
"CO": {"weV": 0.395819, "aeV": 0.289317, "weC": 0.113108, "cnc": 311.3},
"sht": {"hmd": 82.4, "tmp": 12.6}}}

DOCUMENT EXAMPLE - OUTPUT
{"rec": "2018-03-27T12:11:21.028+00:00", "val": {"CO": {"cnc": {"src": 262.1, "med": 368.0}}}}

RESOURCES
Median filter
https://en.wikipedia.org/wiki/Median_filter
"""

import sys

from scs_analysis.cmd.cmd_sample_median import CmdSampleMedian

from scs_core.data.json import JSONify
from scs_core.data.median_filter import MedianFilter
from scs_core.data.path_dict import PathDict


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    document_count = 0
    processed_count = 0

    # ----------------------------------------------------------------------------------------------------------------
    # cmd...

    cmd = CmdSampleMedian()

    if not cmd.is_valid():
        cmd.print_help(sys.stderr)
        exit(2)

    if cmd.verbose:
        print("sample_median: %s" % cmd, file=sys.stderr)


    # ----------------------------------------------------------------------------------------------------------------
    # resources...

    median_filter = MedianFilter(cmd.window)

    if cmd.verbose:
        print("sample_median: %s" % median_filter, file=sys.stderr)
        sys.stderr.flush()

    try:
        # ------------------------------------------------------------------------------------------------------------
        # run...

        for line in sys.stdin:
            datum = PathDict.construct_from_jstr(line)

            if datum is None:
                continue

            document_count += 1

            value = datum.node(cmd.path)

            if value is None:
                continue

            target = PathDict()

            if datum.has_path('rec'):
                target.copy(datum, 'rec')

            target.append(cmd.path + '.src', value)
            target.append(cmd.path + '.med', round(median_filter.compute(value), cmd.precision))

            print(JSONify.dumps(target.node()))
            sys.stdout.flush()

            processed_count += 1


    # ----------------------------------------------------------------------------------------------------------------
    # end...

    except KeyboardInterrupt:
        print(file=sys.stderr)

    except KeyError as ex:
        print("sample_median: %s" % repr(ex), file=sys.stderr)
        exit(1)

    finally:
        if cmd.verbose:
            print("sample_median: documents: %d processed: %d" % (document_count, processed_count), file=sys.stderr)



"""
Created on 2 Nov 2019

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis
"""

import optparse

from glob import glob

from scs_analysis import version
from scs_core.data.model_delta import ModelDelta


# --------------------------------------------------------------------------------------------------------------------

class CmdCollationSummary(object):
    """unix command line handler"""

    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self):
        """
        Constructor
        """
        self.__parser = optparse.OptionParser(usage="%prog  -f FILE_PREFIX -i IND_PATH "
                                                    "[-p IND_PRECISION DEP_PRECISION] [-v] DEP_PATH_1 [..DEP_PATH_N]",
                                              version=version())

        # input...
        self.__parser.add_option("--file-prefix", "-f", type="string", action="store", dest="file_prefix",
                                 help="file prefix for collated CSVs")

        self.__parser.add_option("--ind-path", "-i", type="string", action="store", dest="ind_path",
                                 help="path to independent variable")

        # output...
        self.__parser.add_option("--prec", "-p", type="int", nargs=2, action="store", dest="precisions",
                                 help="precision for independent and dependent variables (default 1, 3 decimals)")

        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    def is_valid(self):
        if self.file_prefix is None or self.ind_path is None or len(self.dep_paths) < 1:
            return False

        return True


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def file_prefix(self):
        return self.__opts.file_prefix


    @property
    def filenames(self):
        return [] if self.file_prefix is None else glob(self.file_prefix + '*.csv')


    @property
    def ind_path(self):
        return self.__opts.ind_path


    @property
    def ind_prec(self):
        return ModelDelta.DEFAULT_INDEPENDENT_PREC if self.__opts.precisions is None else self.__opts.precisions[0]


    @property
    def dep_prec(self):
        return ModelDelta.DEFAULT_DEPENDENT_PREC if self.__opts.precisions is None else self.__opts.precisions[1]


    @property
    def verbose(self):
        return self.__opts.verbose


    @property
    def dep_paths(self):
        return self.__args


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdCollationSummary:{file_prefix:%s, ind_path:%s, precisions:%s, verbose:%s, dep_paths:%s}" % \
               (self.file_prefix, self.ind_path, self.__opts.precisions, self.verbose, self.dep_paths)
